# Standard Library Imports
from typing import Any, Tuple

def get_hazard_rate(scale: float, location: float = ...) -> float: ...
def get_mtbf(rate: float, location: float = ...) -> float: ...
def get_survival(scale: float, time: float, location: float = ...) -> float: ...
def do_fit(data: Any, **kwargs: Any) -> Tuple[float, float]: ...



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel

class ICDesignElectricInputPanel(RAMSTKFixedPanel):
    _dic_technology: Dict[int, List[List[str]]]
    _dic_types: Dict[int, List[List[str]]]
    _select_msg: str
    _tag: str
    _title: str
    cmbApplication: RAMSTKComboBox
    cmbConstruction: RAMSTKComboBox
    cmbECC: RAMSTKComboBox
    cmbManufacturing: RAMSTKComboBox
    cmbPackage: RAMSTKComboBox
    cmbQuality: RAMSTKComboBox
    cmbTechnology: RAMSTKComboBox
    cmbType: RAMSTKComboBox
    txtArea: RAMSTKEntry
    txtFeatureSize: RAMSTKEntry
    txtNActivePins: RAMSTKEntry
    txtNCycles: RAMSTKEntry
    txtNElements: RAMSTKEntry
    txtOperatingLife: RAMSTKEntry
    txtThetaJC: RAMSTKEntry
    txtVoltageESD: RAMSTKEntry
    txtYearsInProduction: RAMSTKEntry
    _hazard_rate_method_id: int
    _quality_id: int
    dic_attribute_widget_map: Dict[str, List[Any]]
    category_id: int
    subcategory_id: int
    def __init__(self) -> None: ...
    def do_load_comboboxes(self, subcategory_id: int) -> None: ...
    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None: ...
    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def _do_load_application_combo(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_dram(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_eeprom(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_gaas(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_linear(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_logic(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_microprocessor_microcontroller(
        self, attributes: Dict[str, Any]
    ) -> None: ...
    def __do_load_pal_pla(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_rom(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_sram(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_vhsic_vlsi(self, attributes: Dict[str, Any]) -> None: ...
    def __do_set_dram_sensitive(self) -> None: ...
    def __do_set_eeprom_sensitive(self) -> None: ...
    def __do_set_gaas_sensitive(self) -> None: ...
    def __do_set_linear_sensitive(self) -> None: ...
    def __do_set_logic_sensitive(self) -> None: ...
    def __do_set_microprocessor_microcontroller_sensitive(self) -> None: ...
    def __do_set_pal_pla_sensitive(self) -> None: ...
    def __do_set_rom_sensitive(self) -> None: ...
    def __do_set_sram_sensitive(self) -> None: ...
    def __do_set_vhsic_vlsi_sensitive(self) -> None: ...



# Standard Library Imports
import gettext
from typing import Any, Callable, List

_ = gettext.gettext

def date_to_ordinal(date: str) -> int: ...
def deprecated(func: Callable) -> Callable: ...
def dir_exists(directory: str) -> bool: ...
def file_exists(_file: str) -> bool: ...
def none_to_default(field: Any, default: Any) -> Any: ...
def none_to_string(string: None) -> str: ...
def ordinal_to_date(ordinal: int) -> str: ...
def split_string(string: str) -> List[str]: ...
def boolean_to_integer(boolean: bool) -> int: ...
def integer_to_boolean(integer: int) -> bool: ...
def string_to_boolean(string: str) -> bool: ...
def get_install_prefix() -> str: ...



# Third Party Imports
from sqlalchemy.engine import Engine as Engine
from sqlalchemy.orm import scoped_session as scoped_session

# RAMSTK Package Imports
from ramstk.models import RAMSTKActionRecord as RAMSTKActionRecord
from ramstk.models import RAMSTKAllocationRecord as RAMSTKAllocationRecord
from ramstk.models import RAMSTKCauseRecord as RAMSTKCauseRecord
from ramstk.models import RAMSTKControlRecord as RAMSTKControlRecord
from ramstk.models import RAMSTKDesignElectricRecord as RAMSTKDesignElectricRecord
from ramstk.models import RAMSTKDesignMechanicRecord as RAMSTKDesignMechanicRecord
from ramstk.models import RAMSTKEnvironmentRecord as RAMSTKEnvironmentRecord
from ramstk.models import RAMSTKFailureDefinitionRecord as RAMSTKFailureDefinitionRecord
from ramstk.models import RAMSTKFunctionRecord as RAMSTKFunctionRecord
from ramstk.models import RAMSTKHardwareRecord as RAMSTKHardwareRecord
from ramstk.models import RAMSTKHazardRecord as RAMSTKHazardRecord
from ramstk.models import RAMSTKMechanismRecord as RAMSTKMechanismRecord
from ramstk.models import RAMSTKMilHdbk217FRecord as RAMSTKMilHdbk217FRecord
from ramstk.models import RAMSTKMissionPhaseRecord as RAMSTKMissionPhaseRecord
from ramstk.models import RAMSTKMissionRecord as RAMSTKMissionRecord
from ramstk.models import RAMSTKModeRecord as RAMSTKModeRecord
from ramstk.models import RAMSTKNSWCRecord as RAMSTKNSWCRecord
from ramstk.models import RAMSTKOpLoadRecord as RAMSTKOpLoadRecord
from ramstk.models import RAMSTKOpStressRecord as RAMSTKOpStressRecord
from ramstk.models import RAMSTKProgramInfoRecord as RAMSTKProgramInfoRecord
from ramstk.models import RAMSTKProgramStatusRecord as RAMSTKProgramStatusRecord
from ramstk.models import RAMSTKReliabilityRecord as RAMSTKReliabilityRecord
from ramstk.models import RAMSTKRequirementRecord as RAMSTKRequirementRecord
from ramstk.models import RAMSTKRevisionRecord as RAMSTKRevisionRecord
from ramstk.models import RAMSTKSimilarItemRecord as RAMSTKSimilarItemRecord
from ramstk.models import RAMSTKStakeholderRecord as RAMSTKStakeholderRecord
from ramstk.models import RAMSTKTestMethodRecord as RAMSTKTestMethodRecord
from ramstk.models import RAMSTKValidationRecord as RAMSTKValidationRecord

def do_make_programdb_tables(engine: Engine) -> None: ...
def do_create_program_db(engine: Engine, session: scoped_session) -> None: ...



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKHazardRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Dict[str, Any]
    __tablename__: str
    __table_args__: Dict[str, bool]
    revision_id: int
    function_id: int
    hazard_id: int
    potential_hazard: str
    potential_cause: str
    assembly_effect: str
    assembly_severity: str
    assembly_probability: str
    assembly_hri: int
    assembly_mitigation: str
    assembly_severity_f: str
    assembly_probability_f: str
    assembly_hri_f: int
    function_1: str
    function_2: str
    function_3: str
    function_4: str
    function_5: str
    remarks: str
    result_1: float
    result_2: float
    result_3: float
    result_4: float
    result_5: float
    system_effect: str
    system_severity: str
    system_probability: str
    system_hri: int
    system_mitigation: str
    system_severity_f: str
    system_probability_f: str
    system_hri_f: int
    user_blob_1: str
    user_blob_2: str
    user_blob_3: str
    user_float_1: float
    user_float_2: float
    user_float_3: float
    user_int_1: int
    user_int_2: int
    user_int_3: int
    def get_attributes(self) -> Dict[str, Any]: ...



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.assistants import EditFunction as EditFunction
from ramstk.views.gtk3.widgets import RAMSTKPanel as RAMSTKPanel
from ramstk.views.gtk3.widgets import RAMSTKWorkView as RAMSTKWorkView

# RAMSTK Local Imports
from . import SimilarItemMethodPanel as SimilarItemMethodPanel
from . import SimilarItemTreePanel as SimilarItemTreePanel

class SimilarItemWorkView(RAMSTKWorkView):
    _tag: str
    _tablabel: str
    _tabtooltip: str
    _lst_mnu_labels: List[str]
    _lst_tooltips: List[str]
    _pnlMethod: RAMSTKPanel
    _pnlPanel: RAMSTKPanel
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    _record_id: int
    _parent_id: int
    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None: ...
    def _do_request_calculate(self, __button: Gtk.ToolButton) -> None: ...
    def _do_request_edit_function(self, __button: Gtk.ToolButton) -> None: ...
    def _do_request_rollup(self, __button: Gtk.ToolButton) -> None: ...
    def __make_ui(self) -> None: ...



# Standard Library Imports
from typing import Any, Dict, Tuple

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.views.gtk3 import Gdk as Gdk
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _

# RAMSTK Local Imports
from .button import do_make_buttonbox as do_make_buttonbox
from .dialog import RAMSTKMessageDialog as RAMSTKMessageDialog
from .label import RAMSTKLabel as RAMSTKLabel
from .panel import RAMSTKPanel as RAMSTKPanel
from .treeview import RAMSTKTreeView as RAMSTKTreeView

class RAMSTKBaseView(Gtk.HBox):
    _pixbuf: bool = ...
    dic_tab_position: Any = ...
    RAMSTK_USER_CONFIGURATION: RAMSTKUserConfiguration = ...
    RAMSTK_LOGGER: Any = ...
    _dic_icons: Any = ...
    _lst_callbacks: Any = ...
    _lst_icons: Any = ...
    _lst_mnu_labels: Any = ...
    _lst_tooltips: Any = ...
    _lst_col_order: Any = ...
    _lst_handler_id: Any = ...
    _lst_layouts: Any = ...
    _img_tab: Any = ...
    _mission_time: Any = ...
    _notebook: Any = ...
    _parent_id: int = ...
    _pnlPanel: Any = ...
    _record_id: int = ...
    _revision_id: int = ...
    _tree_loaded: bool = ...
    fmt: Any = ...
    hbx_tab_label: Any = ...
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def do_embed_treeview_panel(self) -> None: ...
    def do_make_layout(self) -> None: ...
    def do_make_layout_lr(self) -> Gtk.HPaned: ...
    def do_make_layout_lrr(self) -> Tuple[Gtk.HPaned, Gtk.VPaned]: ...
    def do_make_layout_llr(self) -> Tuple[Gtk.HPaned, Gtk.VPaned]: ...
    def do_make_layout_llrr(self) -> Tuple[Gtk.VPaned, Gtk.VPaned]: ...
    def do_raise_dialog(self, **kwargs: Any) -> RAMSTKMessageDialog: ...
    def do_request_delete(self, __button: Gtk.ToolButton) -> None: ...
    def do_request_insert(self, **kwargs: Any) -> None: ...
    def do_request_insert_child(self, __button: Gtk.ToolButton) -> Any: ...
    def do_request_insert_sibling(self, __button: Gtk.ToolButton) -> Any: ...
    def do_request_update(self, __button: Gtk.ToolButton) -> None: ...
    def do_request_update_all(self, __button: Gtk.ToolButton) -> None: ...
    def do_set_cursor(self, cursor: Gdk.CursorType) -> None: ...
    def do_set_cursor_active(self, tree: treelib.Tree = ...) -> None: ...
    def do_set_cursor_active_on_fail(self, error_message: str = ...) -> None: ...
    def do_set_cursor_busy(self) -> None: ...
    def make_tab_label(self, **kwargs: Dict[str, Any]) -> None: ...
    def make_toolbuttons(self, **kwargs: Dict[str, Any]) -> None: ...
    def on_button_press(
        self, treeview: RAMSTKTreeView, event: Gdk.EventButton
    ) -> None: ...
    def on_insert(self, node_id: int = ..., tree: treelib.Tree = ...) -> None: ...
    def on_select_revision(self, attributes: Dict[str, Any]) -> None: ...
    def __set_icons(self) -> Dict[str, str]: ...

class RAMSTKListView(RAMSTKBaseView):
    tab_label: Any = ...
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def do_request_update_all(self, __button: Gtk.ToolButton) -> None: ...
    def make_ui(self) -> None: ...

class RAMSTKModuleView(RAMSTKBaseView):
    _dic_key_index: Any = ...
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def make_ui(self) -> None: ...

class RAMSTKWorkView(RAMSTKBaseView):
    _lst_widgets: Any = ...
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...



# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hazard_analysis.__init__.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Hazard Analysis Views."""

# RAMSTK Local Imports
from .panel import HazardsTreePanel
from .view import HazardsWorkView



# pylint: disable=unused-import, missing-docstring
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.preferences.view.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK GTK3 Preferences Views."""

# Standard Library Imports
from datetime import datetime
from shutil import copyfile
from typing import Any, Dict

# Third Party Imports
import toml
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.utilities import integer_to_boolean
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKBaseView, RAMSTKLabel

# RAMSTK Local Imports
from . import (
    GeneralPreferencesPanel,
    LookFeelPreferencesPanel,
    TreeLayoutPreferencesPanel,
)


class PreferencesDialog(RAMSTKBaseView):
    """Assistant to provide a GUI to set various RAMSTK config preferences.

    RAMSTK preferences are stored in the RAMSTK Site database and the user's Site
    configuration file and Program configuration file. Configurations preferences
    are stored in Site.conf or RAMSTK.conf in each user's $HOME/.config/RAMSTK
    directory and are applicable only to that specific user.  Configuration
    preferences are edited with the Preferences assistant.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag = "preferences"
    _pixbuf: bool = True
    _tablabel: str = _("")
    _tabtooltip: str = _("")

    # Define public dict class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize an instance of the Preferences assistant.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_callbacks = [self._do_request_update, self._cancel]
        self._lst_icons = ["save", "cancel"]
        self._lst_tooltips = [
            _(
                f"Save changes to RAMSTK program configuration file "
                f"{configuration.RAMSTK_CONF_DIR}/RAMSTK.toml."
            ),
            _("Quit the RAMSTK preferences assistant without saving."),
        ]

        # Initialize private scalar attributes.
        self._pnlGeneralPreferences: GeneralPreferencesPanel = GeneralPreferencesPanel()
        self._pnlLookFeel: LookFeelPreferencesPanel = LookFeelPreferencesPanel()
        self._pnlTreeViewLayout: TreeLayoutPreferencesPanel = (
            TreeLayoutPreferencesPanel()
        )

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        pub.sendMessage(
            "request_load_preferences",
            configuration=self.RAMSTK_USER_CONFIGURATION,
        )

    def _cancel(self, __button: Gtk.Button) -> None:
        """Quit the preferences Gtk.Assistant().

        :param __button: the Gtk.Button() that called this method.
        :return: None
        """
        _parent = self.get_parent()
        _parent.destroy()

    def _do_request_update(self, __button: Gtk.Button) -> None:
        """Request to update the user and program preferences.

        :param __button: the Gtk.Button() that called this method.
        :return: None
        """
        _conf_file = self.RAMSTK_USER_CONFIGURATION.RAMSTK_CONF_DIR + "/RAMSTK.toml"
        copyfile(_conf_file, _conf_file + "_bak")
        self.RAMSTK_USER_CONFIGURATION.set_user_configuration()

        try:
            self._do_save_tree_layout()
        # This happens when no format file was edited.
        except FileNotFoundError:
            pass

    def _do_save_tree_layout(self) -> None:
        """Save the Module View RAMSTKTreeView() layout file.

        :return: None
        """
        _layout: Dict[str, Any] = {
            "pixbuf": "False",
            "defaulttitle": {},
            "usertitle": {},
            "datatype": {},
            "position": {},
            "widget": {},
            "editable": {},
            "visible": {},
            "key": {},
        }

        copyfile(
            self._pnlTreeViewLayout.fmt_file, self._pnlTreeViewLayout.fmt_file + "_bak"
        )

        # Get the format file for the Gtk.TreeView to be edited.  Make a
        # backup copy by appending the current date.
        _now = datetime.today().strftime("%Y%m%d")
        _bak_file = f"{self._pnlTreeViewLayout.fmt_file[:-5]}_bak_{_now}.toml"
        copyfile(self._pnlTreeViewLayout.fmt_file, _bak_file)

        # Open the format file for writing.
        with open(self._pnlTreeViewLayout.fmt_file, "w", encoding="utf-8") as _file:
            _model = self._pnlTreeViewLayout.tvwTreeView.get_model()
            _row = _model.get_iter_first()
            while _row is not None:
                _key = _model.get_value(_row, 8)
                _layout["defaulttitle"][_key] = _model.get_value(_row, 0)
                _layout["usertitle"][_key] = _model.get_value(_row, 1)
                _layout["position"][_key] = _model.get_value(_row, 2)
                _layout["editable"][_key] = integer_to_boolean(
                    _model.get_value(_row, 3)
                )
                _layout["visible"][_key] = integer_to_boolean(_model.get_value(_row, 4))
                _layout["datatype"][_key] = _model.get_value(_row, 5)
                _layout["widget"][_key] = _model.get_value(_row, 6)
                _layout["key"][_key] = _model.get_value(_row, 7)

                _row = _model.iter_next(_row)

            toml.dump(_layout, _file)

    def __make_ui(self) -> None:
        """Build the user interface.

        :return: None
        :rtype: None
        """
        super().do_make_layout()

        _label = RAMSTKLabel(_("General &amp; Directories"))
        _label.do_set_properties(
            height=30,
            width=-1,
            justify=Gtk.Justification.CENTER,
            tooltip=_(
                "Allows setting general preferences for the open RAMSTK program."
            ),
        )
        self._notebook.insert_page(
            self._pnlGeneralPreferences, tab_label=_label, position=-1
        )

        _label = RAMSTKLabel(_("Look &amp; Feel"))
        _label.do_set_properties(
            height=30,
            width=-1,
            justify=Gtk.Justification.CENTER,
            tooltip=_("Allows setting color and other preferences."),
        )
        self._notebook.insert_page(self._pnlLookFeel, tab_label=_label, position=-1)

        _label = RAMSTKLabel(_("Tree View Layout"))
        _label.do_set_properties(
            height=30,
            width=-1,
            justify=Gtk.Justification.CENTER,
            tooltip=_("Allows setting tree view layout preferences."),
        )
        self._notebook.insert_page(
            self._pnlTreeViewLayout, tab_label=_label, position=-1
        )

        self.pack_end(self._notebook, True, True, 0)

        self.show_all()



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.components.meter.py is part of the RAMSTK
#       Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Meter Input Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKFixedPanel


class MeterDesignElectricInputPanel(RAMSTKFixedPanel):
    """Display Meter assessment input attribute data in the RAMSTK Work Book.

    The Meter assessment input view displays all the assessment inputs for
    the selected Meter item.  This includes, currently, inputs for
    MIL-HDBK-217FN2.  The attributes of a Meter assessment input view are:

    :cvar dict _dic_quality: dictionary of meter quality levels.  Key is
        meter subcategory ID; values are lists of quality levels.
    :cvar dict _dic_type: dictionary of meter types.  Key is meter
        subcategory ID; values are lists of types.
    :cvar dict _dic_specification: dictionary of meter MIL-SPECs.  Key is
        meter tye ID; values are lists of specifications.
    :cvar dict _dic_insert: dictionary of meter insert materials.  First
        key is meter type ID, second key is meter specification ID; values are
        lists of insert materials.

    :ivar cmbApplication: select and display the application of the meter.
    :ivar cmbType: select and display the type of meter.
    """

    # Define private dict class attributes.

    # Quality levels; key is the subcategory ID.
    _dic_quality: Dict[int, List[List[str]]] = {
        2: [["MIL-SPEC"], [_("Lower")]],
        1: [["MIL-SPEC"], [_("Lower")]],
    }
    # Meter types; key is the subcategory ID.
    _dic_types: Dict[int, List[List[str]]] = {
        1: [[_("AC")], [_("Inverter Driver")], [_("Commutator DC")]],
        2: [[_("Direct Current")], [_("Alternating Current")]],
    }

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "selected_hardware"
    _tag: str = "design_electric"
    _title: str = _("Meter Design Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Meter assessment input view."""
        super().__init__()

        # Initialize widgets.
        self.cmbApplication: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbQuality: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbType: RAMSTKComboBox = RAMSTKComboBox()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._hazard_rate_method_id: int = 0
        self._quality_id: int = 0

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "quality_id": [
                32,
                self.cmbQuality,
                "changed",
                super().on_changed_combo,
                "wvw_editing_reliability",
                0,
                {
                    "tooltip": _("The quality level of the meter."),
                },
                _("Quality Level:"),
                "gint",
            ],
            "type_id": [
                48,
                self.cmbType,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The type of meter."),
                },
                _("Meter Type:"),
                "gint",
            ],
            "application_id": [
                2,
                self.cmbApplication,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The application of the panel meter."),
                },
                _("Meter Function:"),
                "gint",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.category_id: int = 0
        self.subcategory_id: int = 0

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_load_comboboxes,
            "changed_subcategory",
        )
        pub.subscribe(
            self._do_set_reliability_attributes,
            "succeed_get_reliability_attributes",
        )

    def do_load_comboboxes(self, subcategory_id: int) -> None:
        """Load the meter assessment input RAMSTKComboBox()s.

        :param subcategory_id: the subcategory ID of the selected meter.
        :return: None
        :rtype: None
        """
        self.subcategory_id = subcategory_id

        # Load the quality level RAMSTKComboBox().
        if self._hazard_rate_method_id == 1:
            _data = [["MIL-SPEC"], [_("Lower")]]
        else:
            try:
                _data = self._dic_quality[self.subcategory_id]
            except KeyError:
                _data = []
        self.cmbQuality.do_load_combo(_data, signal="changed")

        # Load the meter application RAMSTKComboBox().
        self.cmbApplication.do_load_combo(
            [[_("Ammeter")], [_("Voltmeter")], [_("Other")]], signal="changed"
        )

        # Load the meter type RAMSTKComboBox().
        try:
            _data = self._dic_types[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbType.do_load_combo(_data, signal="changed")

    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._hazard_rate_method_id = attributes["hazard_rate_method_id"]
            self._quality_id = attributes["quality_id"]

            self.cmbQuality.set_sensitive(True)
            self.cmbQuality.do_update(
                self._quality_id,
                signal="changed",
            )

            pub.sendMessage(
                f"request_get_{self._tag}_attributes",
                node_id=self._record_id,
            )

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for the selected meter.

        :return: None
        :rtype: None
        """
        self.cmbApplication.set_sensitive(False)
        self.cmbType.set_sensitive(True)
        self.cmbType.do_update(
            attributes["type_id"],
            signal="changed",
        )

        if self._hazard_rate_method_id == 2 and self.subcategory_id == 2:
            self.cmbApplication.set_sensitive(True)
            self.cmbApplication.do_update(
                attributes["application_id"],
                signal="changed",
            )



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.failure_definition.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Failure Definition Record Model."""

# Third Party Imports
from sqlalchemy import Column, ForeignKey, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKFailureDefinitionRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class representing ramstk_failure_definition table in RAMSTK Program db.

    This table shares a Many-to-One relationship with ramstk_revision.
    """

    __defaults__ = {"definition": "Failure Definition"}
    __tablename__ = "ramstk_failure_definition"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    definition_id = Column(
        "fld_definition_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )

    definition = Column("fld_definition", String, default=__defaults__["definition"])

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve current values of the RAMSTKFailureDefinition attributes.

        :return: {revision_id, definition_id, definition} pairs.
        :rtype: (int, int, str)
        """
        _attributes = {
            "revision_id": self.revision_id,
            "definition_id": self.definition_id,
            "definition": self.definition,
        }

        return _attributes



# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.exim.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK export-import package."""

# RAMSTK Local Imports
from .export import Export
from .imports import Import, _do_replace_nan, _get_input_value



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.views.gtk3 import GObject as GObject
from ramstk.views.gtk3 import Gtk as Gtk

class RAMSTKBaseBook(Gtk.Notebook):
    RAMSTK_SITE_CONFIGURATION: Any = ...
    dic_tab_position: Any = ...
    dic_handler_id: Any = ...
    RAMSTK_USER_CONFIGURATION: Any = ...
    def __init__(self, configuration: RAMSTKUserConfiguration) -> None: ...
    def _set_properties(self, book: str) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.components.semiconductor.py is part of the
#       RAMSTK Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Semiconductor Input Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKEntry, RAMSTKFixedPanel


class SemiconductorDesignElectricInputPanel(RAMSTKFixedPanel):
    """Display Semiconductor assessment input attribute data.

    The Semiconductor assessment input view displays all the assessment inputs
    for the selected semiconductor.  This includes, currently, inputs for
    MIL-HDBK-217FN2.  The attributes of a Semiconductor assessment input view
    are:

    :cvar dict _dic_applications: dictionary of semiconductor applications.
        Key is semiconductor subcategory ID; values are lists of applications.
    :cvar dict _dic_matchings: dictionary of network matching types.  Key is
        semiconductor subcategory ID; values are lists of network matching
        types.
    :cvar dict _dic_quality: dictionary of semiconductor quality levels.  Key
        is semiconductor subcategory ID; values are lists of quality levels.
    :cvar dict _dic_types: dictionary of semiconductor types.  Key is the
        semiconductor subcategory ID; values are lists of semiconductor types.

    :ivar cmbApplication: select and display the application of the
        semiconductor.
    :ivar cmbConstruction: select and display the construction of the
        semiconductor.
    :ivar cmbMatching: select and display the matching arrangement for the
        semiconductor.
    :ivar cmbPackage: select and display the type of package for the
        semiconductor.
    :ivar cmbType: select and display the type of semiconductor.
    :ivar txtFrequencyOperating: enter and display the operating frequency of
        the semiconductor.
    :ivar txtNElements: enter and display the number of elements in the
        optoelectronic display.
    :ivar txtThetaJC: enter and display the junction-case thermal resistance.
    """

    # Define private dict class attributes.
    _dic_quality: Dict[int, List[List[str]]] = {
        1: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        2: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        3: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        4: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        5: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        6: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")]],
        7: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")]],
        8: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")]],
        9: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")]],
        10: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        11: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        12: [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]],
        13: [
            [_("Hermetic Package")],
            [_("Nonhermetic with Facet Coating")],
            [_("Nonhermetic without Facet Coating")],
        ],
    }
    # Key is subcategory ID; index is type ID.
    _dic_types: Dict[int, List[List[str]]] = {
        1: [
            [_("General Purpose Analog")],
            [_("Switching")],
            [_("Power Rectifier, Fast Recovery")],
            [_("Power Rectifier/Schottky Power Diode")],
            [_("Power Rectifier with High Voltage Stacks")],
            [_("Transient Suppressor/Varistor")],
            [_("Current Regulator")],
            [_("Voltage Regulator and Voltage Reference (Avalanche and Zener)")],
        ],
        2: [
            [_("Si IMPATT (<35 GHz)")],
            [_("Gunn/Bulk Effect")],
            [_("Tunnel and Back (Including Mixers, Detectors)")],
            [_("PIN")],
            [
                _(
                    "Schottky Barrier (Including Detectors) and Point Contact "
                    "(200 MHz < Frequency < 35MHz)"
                )
            ],
            [_("Varactor and Step Recovery")],
        ],
        3: [["NPN/PNP (f < 200MHz)"], [_("Power NPN/PNP (f < 200 MHz)")]],
        4: [["MOSFET"], ["JFET"]],
        7: [[_("Gold Metallization")], [_("Aluminum Metallization")]],
        8: [["GaAs FET (P < 100mW)"], ["GaAs FET (P > 100mW)"]],
        9: [["MOSFET"], ["JFET"]],
        11: [
            [_("Photo-Transistor")],
            [_("Photo-Diode")],
            [_("Photodiode Output, Single Device")],
            [_("Phototransistor Output, Single Device")],
            [_("Photodarlington Output, Single Device")],
            [_("Light Sensitive Resistor, Single Device")],
            [_("Photodiode Output, Dual Device")],
            [_("Phototransistor Output, Dual Device")],
            [_("Photodarlington Output, Dual Device")],
            [_("Light Sensitive Resistor, Dual Device")],
            [_("Infrared Light Emitting Diode (IRLED)")],
            [_("Light Emitting Diode")],
        ],
        12: [[_("Segment Display")], [_("Diode Array Display")]],
        13: [["GaAs/Al GaAs"], ["In GaAs/In GaAsP"]],
    }
    # Key is subcategory ID; index is application ID.
    _dic_applications: Dict[int, List[List[str]]] = {
        2: [
            [_("Varactor, Voltage Control")],
            [_("Varactor, Multiplier")],
            [_("All Other Diodes")],
        ],
        3: [[_("Linear Amplification")], [_("Switching")]],
        4: [
            [_("Linear Amplification")],
            [_("Small Signal Switching")],
            [_("Non-Linear (2W < Pr < 5W)")],
            [_("Non-Linear (5W < Pr < 50W)")],
            [_("Non-Linear (50W < Pr < 250W)")],
            [_("Non-Linear (Pr > 250W)")],
        ],
        7: [["CW"], [_("Pulsed")]],
        8: [[_("All Lower Power and Pulsed")], ["CW"]],
        13: [["CW"], [_("Pulsed")]],
    }
    # Key is subcategory ID; index is matching ID.
    _dic_matchings: Dict[int, List[List[str]]] = {
        7: [[_("Input and Output")], [_("Input Only")], [_("None")]],
        8: [[_("Input and Output")], [_("Input Only")], [_("None")]],
    }

    # Define private list class attributes.
    _lst_packages: List[List[str]] = [
        ["TO-1"],
        ["TO-3"],
        ["TO-5"],
        ["TO-8"],
        ["TO-9"],
        ["TO-12"],
        ["TO-18"],
        ["TO-28"],
        ["TO-33"],
        ["TO-39"],
        ["TO-41"],
        ["TO-44"],
        ["TO-46"],
        ["TO-52"],
        ["TO-53"],
        ["TO-57"],
        ["TO-59"],
        ["TO-60"],
        ["TO-61"],
        ["TO-63"],
        ["TO-66"],
        ["TO-71"],
        ["TO-72"],
        ["TO-83"],
        ["TO-89"],
        ["TO-92"],
        ["TO-94"],
        ["TO-99"],
        ["TO-126"],
        ["TO-127"],
        ["TO-204"],
        ["TO-204AA"],
        ["TO-205AD"],
        ["TO-205AF"],
        ["TO-220"],
        ["DO-4"],
        ["DO-5"],
        ["DO-7"],
        ["DO-8"],
        ["DO-9"],
        ["DO-13"],
        ["DO-14"],
        ["DO-29"],
        ["DO-35"],
        ["DO-41"],
        ["DO-45"],
        ["DO-204MB"],
        ["DO-205AB"],
        ["PA-42A,B"],
        ["PD-36C"],
        ["PD-50"],
        ["PD-77"],
        ["PD-180"],
        ["PD-319"],
        ["PD-262"],
        ["PD-975"],
        ["PD-280"],
        ["PD-216"],
        ["PT-2G"],
        ["PT-2G"],
        ["PT-6B"],
        ["PH-13"],
        ["PH-16"],
        ["PH-56"],
        ["PY-58"],
        ["PY-373"],
    ]

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "selected_hardware"
    _tag: str = "design_electric"
    _title: str = _("Semiconductor Design Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize instance of the Semiconductor assessment input view."""
        super().__init__()

        # Initialize widgets.
        self.cmbApplication: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbConstruction: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbMatching: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbPackage: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbQuality: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbType: RAMSTKComboBox = RAMSTKComboBox()
        self.txtFrequencyOperating: RAMSTKEntry = RAMSTKEntry()
        self.txtNElements: RAMSTKEntry = RAMSTKEntry()
        self.txtThetaJC: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._hazard_rate_method_id: int = 0
        self._quality_id: int = 0

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "quality_id": [
                32,
                self.cmbQuality,
                "changed",
                super().on_changed_combo,
                "wvw_editing_reliability",
                0,
                {
                    "tooltip": _("The quality level of the semiconductor."),
                },
                _("Quality Level:"),
                "gint",
            ],
            "package_id": [
                30,
                self.cmbPackage,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The package type for the semiconductor."),
                },
                _("Package:"),
                "gint",
            ],
            "type_id": [
                48,
                self.cmbType,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The type of semiconductor."),
                },
                _("Type:"),
                "gint",
            ],
            "application_id": [
                2,
                self.cmbApplication,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The application of the semiconductor."),
                },
                _("Application:"),
                "gint",
            ],
            "construction_id": [
                6,
                self.cmbConstruction,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The method of construction of the semiconductor."),
                },
                _("Construction:"),
                "gint",
            ],
            "matching_id": [
                6,
                self.cmbMatching,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The matching network of the semiconductor."),
                },
                _("Matching Network:"),
                "gint",
            ],
            "frequency_operating": [
                2,
                self.txtFrequencyOperating,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The operating frequency of the semiconductor."),
                },
                _("Operating Frequency (GHz):"),
                "gfloat",
            ],
            "n_elements": [
                25,
                self.txtNElements,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _(
                        "The number of characters in the optoelectronic display."
                    ),
                },
                _("Number of Characters:"),
                "gint",
            ],
            "theta_jc": [
                47,
                self.txtThetaJC,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The junction-case thermal resistance of the semiconductor."
                    ),
                },
                "\u03B8<sub>JC</sub>:",
                "gfloat",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.category_id: int = 0
        self.subcategory_id: int = 0

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_load_comboboxes,
            "changed_subcategory",
        )
        pub.subscribe(
            self._do_set_reliability_attributes,
            "succeed_get_reliability_attributes",
        )

    def do_load_comboboxes(self, subcategory_id: int) -> None:
        """Load the Semiconductor RAMSTKComboBox()s.

        :param subcategory_id: the subcategory ID of the selected semiconductor.
        :return: None
        :rtype: None
        """
        self.subcategory_id = subcategory_id

        self.cmbPackage.do_load_combo(self._lst_packages)

        self.__do_load_quality()
        self.__do_load_application()
        self.__do_load_construction()
        self.__do_load_matching()
        self.__do_load_type()

    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._hazard_rate_method_id = attributes["hazard_rate_method_id"]
            self._quality_id = attributes["quality_id"]

            self.cmbQuality.set_sensitive(True)
            self.cmbQuality.do_update(
                self._quality_id,
                signal="changed",
            )

            pub.sendMessage(
                f"request_get_{self._tag}_attributes",
                node_id=self._record_id,
            )

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for the selected semiconductor.

        :return: None
        :rtype: None
        """
        self.cmbPackage.set_sensitive(False)
        self.txtThetaJC.set_sensitive(False)

        self.__do_set_application_sensitive(attributes)
        self.__do_set_construction_sensitive(attributes)
        self.__do_set_elements_sensitive(attributes)
        self.__do_set_matching_sensitive(attributes)
        self.__do_set_op_freq_sensitive(attributes)
        self.__do_set_type_sensitive(attributes)

        if self._hazard_rate_method_id == 2:
            self.cmbPackage.set_sensitive(True)
            self.cmbPackage.do_update(
                attributes["package_id"],
                signal="changed",
            )
            self.txtThetaJC.set_sensitive(True)
            self.txtThetaJC.do_update(
                str(self.fmt.format(attributes["theta_jc"] or 0.0)),
                signal="changed",
            )

    def __do_load_application(self) -> None:
        """Load the application RAMSTKComboBox() with selections.

        :return: None
        :rtype: None
        """
        try:
            _data = self._dic_applications[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbApplication.do_load_combo(_data, signal="changed")

    def __do_load_construction(self) -> None:
        """Load the construction RAMSTKComboBox() with selections.

        :return: None
        :rtype: None
        """
        self.cmbConstruction.do_load_combo(
            [
                [_("Metallurgically Bonded")],
                [_("Non-Metallurgically Bonded and Spring Loaded Contacts")],
            ],
            signal="changed",
        )

    def __do_load_matching(self) -> None:
        """Load the matching RAMSTKComboBox() with selections.

        :return: None
        :rtype: None
        """
        try:
            _data = self._dic_matchings[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbMatching.do_load_combo(_data, signal="changed")

    def __do_load_quality(self) -> None:
        """Load the quality RAMSTKComboBox() with selections.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 1:
            if self.subcategory_id == 13:
                _data = [
                    [_("Hermetic Package")],
                    [_("Nonhermetic with Facet Coating")],
                    [_("Nonhermetic without Facet Coating")],
                ]
            else:
                _data = [["JANTXV"], ["JANTX"], ["JAN"], [_("Lower")], [_("Plastic")]]
        else:
            try:
                _data = self._dic_quality[self.subcategory_id]
            except KeyError:
                _data = []
        self.cmbQuality.do_load_combo(_data, signal="changed")

    def __do_load_type(self) -> None:
        """Load the type RAMSTKComboBox() with selections.

        :return: None
        :rtype: None
        """
        try:
            if self._hazard_rate_method_id == 1 and self.subcategory_id == 11:
                _data = [[_("Photodetector")], [_("Opto-Isolator")], [_("Emitter")]]
            else:
                _data = self._dic_types[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbType.do_load_combo(_data, signal="changed")

    def __do_set_application_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the application RAMSTKComboBox() sensitive or not.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 2 and self.subcategory_id in [
            2,
            3,
            4,
            7,
            8,
            13,
        ]:
            self.cmbApplication.set_sensitive(True)
            self.cmbApplication.do_update(
                attributes["application_id"],
                signal="changed",
            )
        else:
            self.cmbApplication.set_sensitive(False)

    def __do_set_construction_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the construction RAMSTKComboBox() sensitive or not.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 2 and self.subcategory_id in [1, 12]:
            self.cmbConstruction.set_sensitive(True)
            self.cmbConstruction.do_update(
                attributes["construction_id"],
                signal="changed",
            )
        else:
            self.cmbConstruction.set_sensitive(False)

    def __do_set_elements_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the number of elements RAMSTKEntry() sensitive or not.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 2 and self.subcategory_id in [7, 8]:
            self.txtNElements.set_sensitive(True)
            self.txtNElements.do_update(
                str(attributes["n_elements"]),
                signal="changed",
            )
        else:
            self.txtNElements.set_sensitive(False)

    def __do_set_matching_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the matching RAMSTKComboBox() sensitive or not.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 2 and self.subcategory_id in [7, 8]:
            self.cmbMatching.set_sensitive(True)
            self.cmbMatching.do_update(
                attributes["matching_id"],
                signal="changed",
            )
        else:
            self.cmbMatching.set_sensitive(False)

    def __do_set_op_freq_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the operating frequency RAMSTKEntry() sensitive or not.

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 2 and self.subcategory_id in [7, 8]:
            self.txtFrequencyOperating.set_sensitive(True)
            self.txtFrequencyOperating.do_update(
                str(self.fmt.format(attributes["frequency_operating"])),
                signal="changed",
            )
        else:
            self.txtFrequencyOperating.set_sensitive(False)

    def __do_set_type_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set the type RAMSTKComboBox() sensitive or not.

        :return: None
        :rtype: None
        """
        if (
            self._hazard_rate_method_id == 1
            and self.subcategory_id
            in [
                1,
                2,
                3,
                8,
                11,
                13,
            ]
        ) or (
            self._hazard_rate_method_id == 2
            and self.subcategory_id
            in [
                1,
                2,
                4,
                7,
                9,
                11,
                12,
                13,
            ]
        ):
            self.cmbType.set_sensitive(True)
            self.cmbType.do_update(
                attributes["type_id"],
                signal="changed",
            )



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3.milhdbk217f import MilHdbk217FResultPanel
from ramstk.views.gtk3.widgets import RAMSTKEntry

class InductorMilHdbk217FResultPanel(MilHdbk217FResultPanel):
    _dic_part_stress: Dict[int, str]
    _title: str
    txtPiC: RAMSTKEntry
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _do_load_entries(self, attributes: Dict[str, Any]) -> None: ...



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord
from ramstk.utilities import none_to_default as none_to_default

class RAMSTKRequirementRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Dict[str, Any]
    __tablename__: str
    __table_args__: Dict[str, bool]
    revision_id: int
    requirement_id: int
    derived: bool
    description: str
    figure_number: str
    owner: str
    page_number: str
    parent_id: int
    priority: int
    requirement_code: str
    specification: str
    requirement_type: str
    validated: bool
    validated_date: str
    q_clarity_0: bool
    q_clarity_1: bool
    q_clarity_2: bool
    q_clarity_3: bool
    q_clarity_4: bool
    q_clarity_5: bool
    q_clarity_6: bool
    q_clarity_7: bool
    q_clarity_8: bool
    q_complete_0: bool
    q_complete_1: bool
    q_complete_2: bool
    q_complete_3: bool
    q_complete_4: bool
    q_complete_5: bool
    q_complete_6: bool
    q_complete_7: bool
    q_complete_8: bool
    q_complete_9: bool
    q_consistent_1: bool
    q_consistent_2: bool
    q_consistent_3: bool
    q_consistent_4: bool
    q_consistent_5: bool
    q_consistent_6: bool
    q_consistent_7: bool
    q_consistent_8: bool
    q_verifiable_0: bool
    q_verifiable_1: bool
    q_verifiable_2: bool
    q_verifiable_3: bool
    q_verifiable_4: bool
    q_verifiable_5: bool
    def get_attributes(self) -> Dict[str, Any]: ...
    def create_code(self, prefix: str) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.fmea.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Failure Mode and Effects Analysis (FMEA) Package View Model."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub
from treelib import Tree

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseView


class RAMSTKFMEAView(RAMSTKBaseView):
    """Contain the attributes and methods of the FMEA view.

    This class manages the usage profile data from the RAMSTKMode,
    RAMSTKMechanism, RAMSTKCause, RAMSTKControl, and RAMSKTAction table models.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _root = 0
    _tag = "fmea"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a FMEA view model instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.
        self._dic_load_functions = {
            "mode": self._do_load_modes,
            "mechanism": self._do_load_mechanisms,
            "cause": self._do_load_causes,
            "control": self._do_load_controls,
            "action": self._do_load_actions,
        }
        self._dic_trees = {
            "mode": Tree(),
            "mechanism": Tree(),
            "cause": Tree(),
            "control": Tree(),
            "action": Tree(),
        }

        # Initialize private list attributes.
        self._lst_modules = [
            "mode",
            "mechanism",
            "cause",
            "control",
            "action",
        ]

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_set_tree, "succeed_insert_mode")
        pub.subscribe(super().do_set_tree, "succeed_insert_mechanism")
        pub.subscribe(super().do_set_tree, "succeed_insert_cause")
        pub.subscribe(super().do_set_tree, "succeed_insert_control")
        pub.subscribe(super().do_set_tree, "succeed_insert_action")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_modes")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_mechanisms")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_causes")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_controls")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_actions")
        pub.subscribe(super().do_set_tree, "succeed_delete_mode")
        pub.subscribe(super().do_set_tree, "succeed_delete_mechanism")
        pub.subscribe(super().do_set_tree, "succeed_delete_cause")
        pub.subscribe(super().do_set_tree, "succeed_delete_control")
        pub.subscribe(super().do_set_tree, "succeed_delete_action")

    def _do_load_modes(self) -> None:
        """Load the failure modes into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["mode"].all_nodes()[1:]:
            _mode = _node.data["mode"]
            _node_id = f"{_mode.mode_id}"

            self.tree.create_node(
                tag="mode",
                identifier=_node_id,
                parent=self._root,
                data={self._tag: _mode},
            )

            if self._dic_trees["mechanism"].depth() > 0:
                self._dic_load_functions["mechanism"](  # type: ignore
                    _mode.mode_id,
                )

    def _do_load_mechanisms(self, mode_id: int) -> None:
        """Load the failure mechanisms into the tree.

        :param mode_id: the ID of the parent failure mode.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["mechanism"].all_nodes()[1:]:
            _mechanism = _node.data["mechanism"]
            _node_id = f"{mode_id}.{_mechanism.mechanism_id}"

            if _mechanism.mode_id == mode_id:
                self.tree.create_node(
                    tag="mechanism",
                    identifier=_node_id,
                    parent=f"{mode_id}",
                    data={self._tag: _mechanism},
                )

                if self._dic_trees["cause"].depth() > 0:
                    self._dic_load_functions["cause"](  # type: ignore
                        _mechanism.mechanism_id,
                        _node_id,
                    )

    def _do_load_causes(self, mechanism_id: int, parent_id: str) -> None:
        """Load the failure causes into the tree for the passed mechanism ID.

        :param mechanism_id: the failure mechanism ID to add the new operating load.
        :param parent_id: the parent node ID.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["cause"].all_nodes()[1:]:
            _cause = _node.data["cause"]
            _node_id = f"{parent_id}.{_cause.cause_id}"

            if _cause.mechanism_id == mechanism_id:
                self.tree.create_node(
                    tag="cause",
                    identifier=_node_id,
                    parent=parent_id,
                    data={self._tag: _cause},
                )

                if self._dic_trees["control"].depth() > 0:
                    self._dic_load_functions["control"](  # type: ignore
                        _cause.cause_id,
                        _node_id,
                    )

                if self._dic_trees["action"].depth() > 0:
                    self._dic_load_functions["action"](  # type: ignore
                        _cause.cause_id,
                        _node_id,
                    )

    def _do_load_controls(self, cause_id: int, parent_id: str) -> None:
        """Load the FNEA controls into the tree.

        :param cause_id: the ID of the parent failure cause.
        :param parent_id: the parent node ID.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["control"].all_nodes()[1:]:
            _control = _node.data["control"]
            _node_id = f"{parent_id}.{_control.control_id}c"

            if _control.cause_id == cause_id:
                self.tree.create_node(
                    tag="control",
                    identifier=_node_id,
                    parent=parent_id,
                    data={self._tag: _control},
                )

    def _do_load_actions(self, cause_id: int, parent_id: str) -> None:
        """Load the FMEA actions into the tree.

        :param cause_id: the ID of the parent failure cause.
        :param parent_id: the parent node ID.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["action"].all_nodes()[1:]:
            _action = _node.data["action"]
            _node_id = f"{parent_id}.{_action.action_id}a"

            if _action.cause_id == cause_id:
                self.tree.create_node(
                    tag="action",
                    identifier=_node_id,
                    parent=parent_id,
                    data={self._tag: _action},
                )



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKManufacturer.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKManufacturer Table Module."""

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKManufacturer(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_manufacturer in the RAMSTK Common database."""

    __defaults__ = {
        "description": "Manufacturer Description",
        "location": "unknown",
        "cage_code": "CAGE Code",
    }
    __tablename__ = "ramstk_manufacturer"
    __table_args__ = {"extend_existing": True}

    manufacturer_id = Column(
        "fld_manufacturer_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    location = Column("fld_location", String(512), default=__defaults__["location"])
    cage_code = Column("fld_cage_code", String(512), default=__defaults__["cage_code"])

    def get_attributes(self):
        """Retrieve current values of RAMSTKManufacturer data model attributes.

        :return: {manufacturer_id, description, location, cage_code} pairs
        :rtype: dict
        """
        _attributes = {
            "manufacturer_id": self.manufacturer_id,
            "description": self.description,
            "location": self.location,
            "cage_code": self.cage_code,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hardware.components.semiconductor.py is part of the RAMSTK
#       Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Semiconductor Results Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.milhdbk217f import MilHdbk217FResultPanel
from ramstk.views.gtk3.widgets import RAMSTKEntry


class SemiconductorMilHdbk217FResultPanel(MilHdbk217FResultPanel):
    """Display semiconductor assessment results attribute data.

    The semiconductor assessment result view displays all the assessment
    results for the selected semiconductor.  This includes, currently, results
    for MIL-HDBK-217FN2 parts count and part stress methods.  The attributes of
    a semiconductor assessment result view are:

    :ivar txtPiT: displays the temperature factor for the semiconductor.
    :ivar txtPiA: displays the application factor for the semiconductor.
    :ivar txtPiC: displays the construction factor for the semiconductor.
    :ivar txtPiI: displays the forward current factor for the semiconductor.
    :ivar txtPiM: displays the matching network factor for the semiconductor.
    :ivar txtPiP: displays the power degradation factor for the semiconductor.
    :ivar txtPiR: displays the power rating factor for the semiconductor.
    :ivar txtPiS: displays the electrical stress factor for the semiconductor.
    """

    _lambda_p = '<span foreground="blue">\u03BB<sub>p</sub> = '
    _function_1 = (
        "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>Q</sub>"
        "\u03C0<sub>E</sub></span>"
    )

    # Define private dict class attributes.
    _dic_part_stress: Dict[int, str] = {
        1: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>S</sub>\u03C0<sub>Q"
        "</sub>\u03C0<sub>E</sub></span>",
        2: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>A</sub>\u03C0<sub>R"
        "</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
        3: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>A</sub>\u03C0<sub>R"
        "</sub>\u03C0<sub>S</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
        4: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>A</sub>\u03C0<sub>Q"
        "</sub>\u03C0<sub>E</sub></span>",
        5: _lambda_p + _function_1,
        6: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>R</sub>\u03C0<sub>S"
        "</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
        7: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>A</sub>\u03C0<sub>M"
        "</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
        8: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>A</sub>\u03C0<sub>M"
        "</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
        9: _lambda_p + _function_1,
        10: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>R</sub>\u03C0<sub>S"
        "</sub>\u03C0<sub>Q</sub>\u03C0<sub>E</sub></span>",
        11: _lambda_p + _function_1,
        12: _lambda_p + _function_1,
        13: _lambda_p
        + "\u03BB<sub>b</sub>\u03C0<sub>T</sub>\u03C0<sub>Q</sub>\u03C0<sub>I"
        "</sub>\u03C0<sub>A</sub>\u03C0<sub>P</sub>\u03C0<sub>E</sub></span> ",
    }

    # Define private list class attributes.

    # Define private scalar class attributes.
    _title: str = _("Semiconductor MIL-HDBK-217F Results")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize instance of the Semiconductor assessment result view."""
        super().__init__()

        # Initialize widgets.
        self.txtPiA: RAMSTKEntry = RAMSTKEntry()
        self.txtPiC: RAMSTKEntry = RAMSTKEntry()
        self.txtPiI: RAMSTKEntry = RAMSTKEntry()
        self.txtPiM: RAMSTKEntry = RAMSTKEntry()
        self.txtPiP: RAMSTKEntry = RAMSTKEntry()
        self.txtPiR: RAMSTKEntry = RAMSTKEntry()
        self.txtPiS: RAMSTKEntry = RAMSTKEntry()
        self.txtPiT: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "hazard_rate_model": [
                13,
                self.lblModel,
                "",
                None,
                "",
                "",
                {
                    "tooltip": _(
                        "The assessment model used to calculate the semiconductor "
                        "hazard rate."
                    ),
                },
                "",
            ],
            "lambda_b": [
                23,
                self.txtLambdaB,
                "",
                None,
                "",
                0.0,
                {
                    "tooltip": _("The base hazard rate for the semiconductor."),
                },
                "\u03BB<sub>b</sub>:",
            ],
            "pi_q": [
                30,
                self.txtPiQ,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The quality factor for the semiconductor."),
                },
                "\u03C0<sub>Q</sub>:",
            ],
            "pi_e": [
                19,
                self.txtPiE,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The environment factor for the semiconductor."),
                },
                "\u03C0<sub>E</sub>:",
            ],
            "pi_t": [
                33,
                self.txtPiT,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The temperature factor for the semiconductor."),
                },
                "\u03C0<sub>T</sub>:",
            ],
            "pi_a": [
                12,
                self.txtPiA,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The application factor for the semiconductor."),
                },
                "\u03C0<sub>A</sub>:",
            ],
            "pi_c": [
                13,
                self.txtPiC,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _(
                        "The contact construction factor for the semiconductor."
                    ),
                },
                "\u03C0<sub>C</sub>:",
            ],
            "pi_r": [
                31,
                self.txtPiR,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The power rating factor for the semiconductor."),
                },
                "\u03C0<sub>R</sub>:",
            ],
            "pi_m": [
                24,
                self.txtPiM,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The matching network factor for the semiconductor."),
                },
                "\u03C0<sub>M</sub>:",
            ],
            "pi_i": [
                21,
                self.txtPiI,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The forward current factor for the semiconductor."),
                },
                "\u03C0<sub>I</sub>:",
            ],
            "pi_p": [
                28,
                self.txtPiP,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The power degradation factor for the semiconductor."),
                },
                "\u03C0<sub>P</sub>:",
            ],
            "pi_s": [
                31,
                self.txtPiS,
                "",
                None,
                "",
                1.0,
                {
                    "tooltip": _("The electrical stress factor for the semiconductor."),
                },
                "\u03C0<sub>S</sub>:",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        super().do_set_properties()
        super().do_make_panel()

        # Subscribe to PyPubSub messages.

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Load the semiconductor assessment results page.

        :param attributes: the attributes dictionary for the selected
            Semiconductor.
        :return: None
        :rtype: None
        """
        super().do_load_entries(attributes)

        self.txtPiA.set_sensitive(False)
        self.txtPiC.set_sensitive(False)
        self.txtPiI.set_sensitive(False)
        self.txtPiM.set_sensitive(False)
        self.txtPiP.set_sensitive(False)
        self.txtPiR.set_sensitive(False)
        self.txtPiS.set_sensitive(False)
        self.txtPiT.set_sensitive(False)

        if self.category_id == 2 and self._hazard_rate_method_id == 2:
            self.txtPiA.do_update(str(self.fmt.format(attributes["piA"])))
            self.txtPiC.do_update(str(self.fmt.format(attributes["piC"])))
            self.txtPiI.do_update(str(self.fmt.format(attributes["piI"])))
            self.txtPiM.do_update(str(self.fmt.format(attributes["piM"])))
            self.txtPiP.do_update(str(self.fmt.format(attributes["piP"])))
            self.txtPiR.do_update(str(self.fmt.format(attributes["piR"])))
            self.txtPiS.do_update(str(self.fmt.format(attributes["piS"])))
            self.txtPiT.do_update(str(self.fmt.format(attributes["piT"])))



# Standard Library Imports
from typing import Any, Callable, Dict, List, Tuple

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gdk as Gdk
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKButton as RAMSTKButton
from ramstk.views.gtk3.widgets import RAMSTKCheckButton as RAMSTKCheckButton
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKDateSelect as RAMSTKDateSelect
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel
from ramstk.views.gtk3.widgets import RAMSTKTextView as RAMSTKTextView
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class RequirementTreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: str
    _dic_row_loader: Dict[str, Callable]
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _on_module_switch(self, module: str = ...) -> None: ...
    _record_id: int
    _parent_id: int
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...
    def __do_load_requirement(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...

class RequirementGeneralDataPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    btnValidateDate: RAMSTKButton
    chkDerived: RAMSTKCheckButton
    chkValidated: RAMSTKCheckButton
    cmbOwner: RAMSTKComboBox
    cmbRequirementType: RAMSTKComboBox
    cmbPriority: RAMSTKComboBox
    txtCode: RAMSTKEntry
    txtFigNum: RAMSTKEntry
    txtName: RAMSTKEntry
    txtPageNum: RAMSTKEntry
    txtSpecification: RAMSTKEntry
    txtValidatedDate: RAMSTKEntry
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def do_load_priorities(self) -> None: ...
    def do_load_requirement_types(
        self, requirement_types: Dict[int, Tuple[str]]
    ) -> None: ...
    def do_load_workgroups(self, workgroups: Dict[int, Tuple[str]]) -> None: ...
    def _do_load_code(self, requirement_code: int) -> None: ...
    @staticmethod
    def _do_select_date(
        __button: RAMSTKButton, __event: Gdk.Event, entry: RAMSTKEntry
    ) -> str: ...

class RequirementClarityPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    chkClarityQ0: RAMSTKCheckButton
    chkClarityQ1: RAMSTKCheckButton
    chkClarityQ2: RAMSTKCheckButton
    chkClarityQ3: RAMSTKCheckButton
    chkClarityQ4: RAMSTKCheckButton
    chkClarityQ5: RAMSTKCheckButton
    chkClarityQ6: RAMSTKCheckButton
    chkClarityQ7: RAMSTKCheckButton
    chkClarityQ8: RAMSTKCheckButton
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...

class RequirementCompletenessPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    chkCompleteQ0: RAMSTKCheckButton
    chkCompleteQ1: RAMSTKCheckButton
    chkCompleteQ2: RAMSTKCheckButton
    chkCompleteQ3: RAMSTKCheckButton
    chkCompleteQ4: RAMSTKCheckButton
    chkCompleteQ5: RAMSTKCheckButton
    chkCompleteQ6: RAMSTKCheckButton
    chkCompleteQ7: RAMSTKCheckButton
    chkCompleteQ8: RAMSTKCheckButton
    chkCompleteQ9: RAMSTKCheckButton
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...

class RequirementConsistencyPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    chkConsistentQ0: RAMSTKCheckButton
    chkConsistentQ1: RAMSTKCheckButton
    chkConsistentQ2: RAMSTKCheckButton
    chkConsistentQ3: RAMSTKCheckButton
    chkConsistentQ4: RAMSTKCheckButton
    chkConsistentQ5: RAMSTKCheckButton
    chkConsistentQ6: RAMSTKCheckButton
    chkConsistentQ7: RAMSTKCheckButton
    chkConsistentQ8: RAMSTKCheckButton
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...

class RequirementVerifiabilityPanel(RAMSTKFixedPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    chkVerifiableQ0: RAMSTKCheckButton
    chkVerifiableQ1: RAMSTKCheckButton
    chkVerifiableQ2: RAMSTKCheckButton
    chkVerifiableQ3: RAMSTKCheckButton
    chkVerifiableQ4: RAMSTKCheckButton
    chkVerifiableQ5: RAMSTKCheckButton
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...



# Standard Library Imports
from typing import Any, Dict, List, Type

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable as RAMSTKBaseTable
from ramstk.models import RAMSTKFailureDefinitionRecord as RAMSTKFailureDefinitionRecord

class RAMSTKFailureDefinitionTable(RAMSTKBaseTable):
    _db_id_colname: str
    _db_tablename: str
    _select_msg: str
    _tag: str
    _lst_id_columns: List[str]
    _record: Type[RAMSTKFailureDefinitionRecord]
    pkey: str
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def do_get_new_record(
        self, attributes: Dict[str, Any]
    ) -> Type[RAMSTKFailureDefinitionRecord]: ...



# Standard Library Imports
from typing import Any, Dict, List, Union

# RAMSTK Package Imports
from ramstk.analyses import dormancy as dormancy
from ramstk.analyses.statistics import exponential as exponential
from ramstk.analyses.statistics import lognormal as lognormal
from ramstk.analyses.statistics import normal as normal
from ramstk.analyses.statistics import weibull as weibull
from ramstk.models import RAMSTKBaseTable as RAMSTKBaseTable
from ramstk.models import RAMSTKReliabilityRecord as RAMSTKReliabilityRecord

class RAMSTKReliabilityTable(RAMSTKBaseTable):
    _db_id_colname: str
    _db_tablename: str
    _select_msg: str
    _tag: str
    _lst_id_columns: List[str]
    _record: RAMSTKReliabilityRecord
    pkey: str
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def do_get_new_record(
        self, attributes: Dict[str, Union[float, int, str]]
    ) -> RAMSTKReliabilityRecord: ...



# Standard Library Imports
from typing import Any, Callable, Dict, List

# Third Party Imports
from treelib import Tree

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseView

class RAMSTKUsageProfileView(RAMSTKBaseView):
    _root: int = ...
    _tag: str = ...
    _dic_load_functions: Dict[str, Callable] = ...
    _dic_trees: Dict[str, Tree] = ...
    _lst_modules: List[str] = ...
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def _do_load_environments(self, phase_id: int, parent_id: str) -> None: ...
    def _do_load_missions(self) -> None: ...
    def _do_load_mission_phases(self, mission_id: int) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.requirement.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Requirement Table Model."""

# Standard Library Imports
import inspect
from typing import Any, Dict, Type

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable, RAMSTKRequirementRecord


class RAMSTKRequirementTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Requirement data manager."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_requirement_id"
    _db_tablename = "ramstk_requirement"
    _select_msg = "selected_revision"
    _tag = "requirement"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a Requirement data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "requirement_id",
            "parent_id",
            "record_id",
        ]

        # Initialize private scalar attributes.
        self._record: Type[RAMSTKRequirementRecord] = RAMSTKRequirementRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "requirement_id"

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            super().do_create_all_codes, "request_create_all_requirement_codes"
        )

        pub.subscribe(self.do_create_code, "request_create_requirement_code")

    def do_create_code(self, node_id: int, prefix: str) -> None:
        """Request to create the requirement code.

        :param node_id: the Requirement ID to create the code for.
        :param prefix: the code prefix to use for the requested code.
        :return: None
        :rtype: None
        """
        try:
            _requirement = self.tree.get_node(node_id).data["requirement"]
            _requirement.create_code(prefix=prefix)

            pub.sendMessage("succeed_create_code")
            pub.sendMessage(
                "succeed_create_requirement_code",
                requirement_code=_requirement.get_attributes()["requirement_code"],
            )
        except (TypeError, AttributeError):
            if node_id != 0:
                _method_name: str = (
                    inspect.currentframe().f_code.co_name  # type: ignore
                )
                pub.sendMessage(
                    "fail_create_requirement_code",
                    error_message=(
                        f"{_method_name}: No data package found for requirement "
                        f"ID {node_id}."
                    ),
                )

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> object:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        self._parent_id = attributes[  # pylint: disable=attribute-defined-outside-init
            "parent_id"
        ]

        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.requirement_id = self.last_id + 1
        _new_record.parent_id = attributes.get("parent_id", 0)

        return _new_record



# -*- coding: utf-8 -*-
#
#       ramstk.models.mechanism.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Failure Mechanism Table Model."""

# Standard Library Imports
from typing import Any, Dict, Type

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.analyses import criticality
from ramstk.models import RAMSTKBaseTable, RAMSTKMechanismRecord


class RAMSTKMechanismTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Mechanism data manager."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_mechanism_id"
    _db_tablename = "ramstk_mechanism"
    _select_msg = "selected_revision"
    _tag = "mechanism"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[str, Any]) -> None:
        """Initialize a Mechanism data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "hardware_id",
            "mode_id",
            "mechanism_id",
        ]

        # Initialize private scalar attributes.
        self._record: Type[RAMSTKMechanismRecord] = RAMSTKMechanismRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "mechanism_id"

        # Subscribe to PyPubSub messages.
        pub.subscribe(self.do_calculate_rpn, "request_calculate_mechanism_rpn")

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> object:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.hardware_id = attributes["hardware_id"]
        _new_record.mode_id = attributes["mode_id"]
        _new_record.mechanism_id = self.last_id + 1

        return _new_record

    def do_calculate_rpn(self, severity: int) -> None:
        """Calculate the risk priority number (RPN) of a hardware item's modes.

            .. note:: the severity (S) value will always be associated with a
                failure mode.

            .. note:: the occurrence (O) and detection (D) values may be
                associated with a failure mechanism or a failure cause.  Typically,
                hardware FMEA use mechanisms and functional FMEA use causes.

        :param severity: the RPN severity value.
        :return: None
        :rtype: None
        """
        _sod = {"rpn_severity": severity, "rpn_occurrence": 10, "rpn_detection": 10}

        for _node in self.tree.all_nodes()[1:]:
            _sod["rpn_occurrence"] = _node.data[self._tag].rpn_occurrence
            _sod["rpn_detection"] = _node.data[self._tag].rpn_detection
            _node.data[self._tag].rpn = criticality.calculate_rpn(_sod)

            _sod["rpn_occurrence"] = _node.data[self._tag].rpn_occurrence_new
            _sod["rpn_detection"] = _node.data[self._tag].rpn_detection_new
            _node.data[self._tag].rpn_new = criticality.calculate_rpn(_sod)

        pub.sendMessage(
            "succeed_calculate_mechanism_rpn",
            tree=self.tree,
        )



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.programdb.RAMSTKOpStress.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2021 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKOpStress Table."""

# Third Party Imports
from sqlalchemy import Column, ForeignKeyConstraint, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKOpStressRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent table ramstk_op_stress in RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_op_load.
    """

    __defaults__ = {
        "description": "",
        "load_history": "",
        "measurable_parameter": "",
        "remarks": "",
    }
    __tablename__ = "ramstk_op_stress"
    __table_args__ = (
        ForeignKeyConstraint(
            [
                "fld_revision_id",
                "fld_hardware_id",
                "fld_mode_id",
                "fld_mechanism_id",
                "fld_load_id",
            ],
            [
                "ramstk_op_load.fld_revision_id",
                "ramstk_op_load.fld_hardware_id",
                "ramstk_op_load.fld_mode_id",
                "ramstk_op_load.fld_mechanism_id",
                "ramstk_op_load.fld_load_id",
            ],
        ),
        {"extend_existing": True},
    )

    revision_id = Column("fld_revision_id", Integer, primary_key=True, nullable=False)
    hardware_id = Column(
        "fld_hardware_id", Integer, primary_key=True, default=-1, nullable=False
    )
    mode_id = Column("fld_mode_id", Integer, primary_key=True, nullable=False)
    mechanism_id = Column("fld_mechanism_id", Integer, primary_key=True, nullable=False)
    load_id = Column(
        "fld_load_id", Integer, primary_key=True, nullable=False, unique=True
    )
    stress_id = Column(
        "fld_stress_id", Integer, primary_key=True, autoincrement=True, nullable=False
    )

    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    load_history = Column(
        "fld_load_history", String(512), default=__defaults__["load_history"]
    )
    measurable_parameter = Column(
        "fld_measurable_parameter",
        String(512),
        default=__defaults__["measurable_parameter"],
    )
    remarks = Column("fld_remarks", String, default=__defaults__["remarks"])

    # Define the relationships to other tables in the RAMSTK Program database.
    op_load = relationship(  # type: ignore
        "RAMSTKOpLoadRecord",
        back_populates="op_stress",
    )

    is_mode = False
    is_mechanism = False
    is_opload = False
    is_opstress = True
    is_testmethod = False

    def get_attributes(self):
        """Retrieve the current values of the Op Stress data model attributes.

        :return: {load_id, stress_id, description, load_history,
                  measurable_parameter, remarks} pairs
        :rtype: tuple
        """
        _attributes = {
            "load_id": self.load_id,
            "stress_id": self.stress_id,
            "description": self.description,
            "load_history": self.load_history,
            "measurable_parameter": self.measurable_parameter,
            "remarks": self.remarks,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.models.pof.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Physics of Failure Package View Model."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub
from treelib import Tree

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseView


class RAMSTKPoFView(RAMSTKBaseView):
    """Contain the attributes and methods of the Physics of Failure (PoF) view.

    This class manages the usage profile data from the RAMSTKMechanism,
    RAMSTKOpLoad, RAMSTKOpStress, and RAMSKTTestMethod table models.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _root = 0
    _tag = "pof"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a PoF view model instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.
        self._dic_load_functions = {
            "mechanism": self._do_load_mechanisms,
            "opload": self._do_load_oploads,
            "opstress": self._do_load_opstress,
            "test_method": self._do_load_test_method,
        }
        self._dic_trees = {
            "mechanism": Tree(),
            "opload": Tree(),
            "opstress": Tree(),
            "test_method": Tree(),
        }

        # Initialize private list attributes.
        self._lst_modules = [
            "mechanism",
            "opload",
            "opstress",
            "test_method",
        ]

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_set_tree, "succeed_insert_mechanism")
        pub.subscribe(super().do_set_tree, "succeed_insert_opload")
        pub.subscribe(super().do_set_tree, "succeed_insert_opstress")
        pub.subscribe(super().do_set_tree, "succeed_insert_test_method")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_mechanisms")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_oploads")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_opstresss")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_test_methods")
        pub.subscribe(super().do_set_tree, "succeed_delete_mechanism")
        pub.subscribe(super().do_set_tree, "succeed_delete_opload")
        pub.subscribe(super().do_set_tree, "succeed_delete_opstress")
        pub.subscribe(super().do_set_tree, "succeed_delete_test_method")

    def _do_load_mechanisms(self) -> None:
        """Load the mechanisms into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["mechanism"].all_nodes()[1:]:
            _mechanism = _node.data["mechanism"]
            _node_id = f"{_mechanism.mechanism_id}"

            self.tree.create_node(
                tag="mechanism",
                identifier=_node_id,
                parent=self._root,
                data={self._tag: _mechanism},
            )

            if self._dic_trees["opload"].depth() > 0:
                self._dic_load_functions["opload"](  # type: ignore
                    _mechanism.mechanism_id,
                )

    def _do_load_oploads(self, mechanism_id: int) -> None:
        """Load the operating loads into the tree for the passed mechanism ID.

        :param mechanism_id: the failure mechanism ID to add the new operating load.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["opload"].all_nodes()[1:]:
            _opload = _node.data["opload"]
            _node_id = f"{mechanism_id}.{_opload.load_id}"

            if _opload.mechanism_id == mechanism_id:
                self.tree.create_node(
                    tag="opload",
                    identifier=_node_id,
                    parent=f"{mechanism_id}",
                    data={self._tag: _opload},
                )

                if self._dic_trees["opstress"].depth() > 0:
                    self._dic_load_functions["opstress"](  # type: ignore
                        _opload.load_id,
                        _node_id,
                    )

                if self._dic_trees["test_method"].depth() > 0:
                    self._dic_load_functions["test_method"](  # type: ignore
                        _opload.load_id,
                        _node_id,
                    )

    def _do_load_opstress(self, load_id: int, parent_id: str) -> None:
        """Load the operating stresses into the tree for the passed load ID.

        :param load_id: the operating load ID to load the operating stresses for.
        :param parent_id: the parent node ID.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["opstress"].all_nodes()[1:]:
            _opstress = _node.data["opstress"]
            _node_id = f"{parent_id}.{_opstress.stress_id}s"

            if _opstress.load_id == load_id:
                self.tree.create_node(
                    tag="opstress",
                    identifier=_node_id,
                    parent=parent_id,
                    data={self._tag: _opstress},
                )

    def _do_load_test_method(self, load_id: int, parent_id: str) -> None:
        """Load the operating stresses into the tree for the passed load ID.

        :param load_id: the operating load ID to load the operating stresses for.
        :param parent_id: the parent node ID.
        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["test_method"].all_nodes()[1:]:
            _test_method = _node.data["test_method"]
            _node_id = f"{parent_id}.{_test_method.test_id}t"

            if _test_method.load_id == load_id:
                self.tree.create_node(
                    tag="test_method",
                    identifier=_node_id,
                    parent=parent_id,
                    data={self._tag: _test_method},
                )



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel

class InductorDesignElectricInputPanel(RAMSTKFixedPanel):
    _dic_insulation: Dict[int, List[List[str]]]
    _dic_quality: Dict[int, List[List[str]]]
    _dic_specifications: Dict[int, List[List[str]]]
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    cmbConstruction: RAMSTKComboBox
    cmbFamily: RAMSTKComboBox
    cmbInsulation: RAMSTKComboBox
    cmbSpecification: RAMSTKComboBox
    txtArea: RAMSTKEntry
    txtWeight: RAMSTKEntry
    _hazard_rate_method_id: int
    _quality_id: int
    dic_attribute_widget_map: Dict[str, List[Any]]
    category_id: int
    subcategory_id: int
    def __init__(self) -> None: ...
    def do_load_comboboxes(self, subcategory_id: int) -> None: ...
    def _do_load_panel(self, attributes: Dict[str, Any]) -> None: ...
    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None: ...
    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None: ...
    def __do_load_family_combobox(self) -> None: ...
    def __do_load_insulation_combobox(self) -> None: ...
    def __do_load_quality_combobox(self) -> None: ...
    def __do_load_specification_combobox(self) -> None: ...



# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKFunctionRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Dict[str, Any]
    __tablename__: str
    __table_args__: Dict[str, bool]
    revision_id: int
    function_id: int
    availability_logistics: float
    availability_mission: float
    cost: float
    function_code: str
    hazard_rate_logistics: float
    hazard_rate_mission: float
    level: int
    mmt: float
    mcmt: float
    mpmt: float
    mtbf_logistics: float
    mtbf_mission: float
    mttr: float
    name: str
    parent_id: int
    remarks: str
    safety_critical: bool
    total_mode_count: int
    total_part_count: int
    type_id: int
    hazard: relationship
    def get_attributes(self) -> Dict[str, Any]: ...



# Standard Library Imports
from typing import Dict, List

# Third Party Imports
import pandas as pd

# RAMSTK Package Imports
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKPlotPanel as RAMSTKPlotPanel

class ProgramStatusPlotPanel(RAMSTKPlotPanel):
    _record_field: str
    _select_msg: str
    _tag: str
    _title: str
    lst_axis_labels: List[str]
    lst_legend: List[str]
    plot_title: str
    def __init__(self) -> None: ...
    def _do_load_panel(self, attributes: Dict[str, pd.DataFrame]) -> None: ...
    def _do_load_assessment_milestones(
        self, assessed: pd.DataFrame, y_max: float
    ) -> None: ...
    def _do_load_plan(self, plan: pd.DataFrame) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.Configuration.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Configuration information and methods for RAMSTK."""

# Standard Library Imports
import gettext
import glob
import sys

# pylint: disable=no-name-in-module
from distutils import dir_util, file_util

# pylint: disable=no-name-in-module
from distutils.errors import DistutilsFileError
from os import environ, makedirs
from typing import Any, Dict, List, Tuple

# Third Party Imports
# noinspection PyPackageRequirements
import toml
from pubsub import pub

# RAMSTK Package Imports
from ramstk.utilities import dir_exists, file_exists, get_install_prefix

_ = gettext.gettext

# Define global list constants.
RAMSTK_ACTIVE_ENVIRONMENTS = [
    [_("Ground, Benign")],
    [_("Ground, Fixed")],
    [_("Ground, Mobile")],
    [_("Naval, Sheltered")],
    [_("Naval, Unsheltered")],
    [_("Airborne, Inhabited, Cargo")],
    [_("Airborne, Inhabited, Fighter")],
    [_("Airborne, Uninhabited, Cargo")],
    [_("Airborne, Uninhabited, Fighter")],
    [_("Airborne, Rotary Wing")],
    [_("Space, Flight")],
    [_("Missile, Flight")],
    [_("Missile, Launch")],
]
RAMSTK_DORMANT_ENVIRONMENTS = [
    [_("Airborne")],
    [_("Ground")],
    [_("Naval")],
    [_("Space")],
]

RAMSTK_ALLOCATION_MODELS = [
    ["Equal Apportionment"],
    ["ARINC Apportionment"],
    ["AGREE Apportionment"],
    ["Feasibility of Objectives"],
    ["Repairable Systems Apportionment"],
]

RAMSTK_HR_TYPES = [
    [_("Assessed")],
    [_("Defined, Hazard Rate")],
    [_("Defined, MTBF")],
    [_("Defined, Distribution")],
]
RAMSTK_HR_MODELS = [
    [_("MIL-HDBK-217F Parts Count")],
    [_("MIL-HDBK-217F Parts Stress")],
    [_("NSWC-11")],
]
RAMSTK_HR_DISTRIBUTIONS = [
    [_("1P Exponential")],
    [_("2P Exponential")],
    [_("Gaussian")],
    [_("Lognormal")],
    [_("2P Weibull")],
    [_("3P Weibull")],
]

RAMSTK_CONTROL_TYPES = [_("Prevention"), _("Detection")]
RAMSTK_COST_TYPES = [[_("Defined")], [_("Calculated")]]
RAMSTK_MTTR_TYPES = [[_("Defined")], [_("Calculated")]]

RAMSTK_CRITICALITY = [
    [
        _("Catastrophic"),
        _(
            "Could result in death, permanent total disability, loss "
            "exceeding $1M, or irreversible severe environmental damage that "
            "violates law or regulation."
        ),
        "I",
        4,
    ],
    [
        _("Critical"),
        _(
            "Could result in permanent partial disability, injuries or "
            "occupational illness that may result in hospitalization of at "
            "least three personnel, loss exceeding $200K but less than $1M, "
            "or reversible environmental damage causing a violation of law or "
            "regulation."
        ),
        "II",
        3,
    ],
    [
        _("Marginal"),
        _(
            "Could result in injury or occupational illness resulting in one "
            "or more lost work days(s), loss exceeding $10K but less than "
            "$200K, or mitigatible environmental damage without violation of "
            "law or regulation where restoration activities can be "
            "accomplished."
        ),
        "III",
        2,
    ],
    [
        _("Negligble"),
        _(
            "Could result in injury or illness not resulting in a lost work "
            "day, loss exceeding $2K but less than $10K, or minimal "
            "environmental damage not violating law or regulation."
        ),
        "IV",
        1,
    ],
]
RAMSTK_FAILURE_PROBABILITY = [
    [_("Level E - Extremely Unlikely"), 1],
    [_("Level D - Remote"), 2],
    [_("Level C - Occasional"), 3],
    [_("Level B - Reasonably Probable"), 4],
    [_("Level A - Frequent"), 5],
]

RAMSTK_SW_DEV_ENVIRONMENTS = [
    [_("Organic"), 1.0, 0.76],
    [_("Semi-Detached"), 1.0, 1.0],
    [_("Embedded"), 1.0, 1.3],
]
RAMSTK_SW_DEV_PHASES = [
    [_("Concept/Planning (PCP)")],
    [_("Requirements Analysis (SRA)")],
    [_("Preliminary Design Review (PDR)")],
    [_("Critical Design Review (CDR)")],
    [_("Test Readiness Review (TRR)")],
    [_("Released")],
]
RAMSTK_SW_LEVELS = [
    [_("Software System"), 0],
    [_("Software Module"), 0],
    [_("Software Unit"), 0],
]
RAMSTK_SW_APPLICATION = [
    [_("Airborne"), 0.0128, 6.28],
    [_("Strategic"), 0.0092, 1.2],
    [_("Tactical"), 0.0078, 13.8],
    [_("Process Control"), 0.0018, 3.8],
    [_("Production Center"), 0.0085, 23.0],
    [_("Developmental"), 0.0123, 132.6],
]
RAMSTK_SW_TEST_METHODS = [
    [
        _("Code Reviews"),
        _(
            "Code review is a systematic examination (often known as peer "
            "review) of computer source code."
        ),
    ],
    [
        _("Error/Anomaly Detection"),
        _(""),
    ],
    [
        _("Structure Analysis"),
        _(""),
    ],
    [
        _("Random Testing"),
        _(""),
    ],
    [
        _("Functional Testing"),
        _(""),
    ],
    [
        _("Branch Testing"),
        _(""),
    ],
]

RAMSTK_LIFECYCLE = [
    [_("Design")],
    [_("Reliability Growth")],
    [_("Reliability Qualification")],
    [_("Production")],
    [_("Storage")],
    [_("Operation")],
    [_("Disposal")],
]
RAMSTK_S_DIST = [
    ["Constant Probability"],
    ["Exponential"],
    ["Gaussian"],
    ["LogNormal"],
    ["Uniform"],
    ["Weibull"],
]


class RAMSTKSiteConfiguration:
    """Class for site-wide RAMSTK configuration settings."""

    def __init__(self) -> None:
        """Initialize the RAMSTK site configuration class."""
        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._INSTALL_PREFIX = get_install_prefix()

        # Initialize public dictionary attributes.
        self.RAMSTK_COM_INFO: Dict[str, str] = {}

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.RAMSTK_COM_BACKEND = ""
        if sys.platform in ["linux", "linux2"]:
            self.RAMSTK_SITE_DIR = self._INSTALL_PREFIX + "/share/RAMSTK"
        elif sys.platform == "win32":
            self.RAMSTK_SITE_DIR = environ["PYTHONPATH"] + "/RAMSTK"

        self.RAMSTK_SITE_CONF = self.RAMSTK_SITE_DIR + "/Site.toml"

    def do_create_site_configuration(self) -> None:
        """Create the default site configuration file.

        :return: None
        :rtype: None
        """
        _dic_site_configuration = {
            "title": "RAMSTK Site Configuration",
            "backend": {
                "dialect": "postgres",
                "host": "",
                "port": "5432",
                "database": "ramstk_common_ramstk",
                "user": "first_run",
                "password": "",
            },
        }

        try:
            with open(self.RAMSTK_SITE_CONF, "w", encoding="utf-8") as _site_conf:
                toml.dump(_dic_site_configuration, _site_conf)
            pub.sendMessage("succeed_create_site_configuration")
        except FileNotFoundError:
            _error_msg = (
                f"Failed to write site configuration file {self.RAMSTK_SITE_CONF}."
            )
            pub.sendMessage("fail_create_site_configuration", error_message=_error_msg)

    def get_site_configuration(self) -> None:
        """Read the site configuration file.

        :return: None
        :rtype: None
        """
        if file_exists(self.RAMSTK_SITE_CONF):
            _config = toml.load(self.RAMSTK_SITE_CONF)

            self.RAMSTK_COM_BACKEND = _config["backend"]["dialect"]
            self.RAMSTK_COM_INFO["dialect"] = _config["backend"]["dialect"]
            self.RAMSTK_COM_INFO["host"] = _config["backend"]["host"]
            self.RAMSTK_COM_INFO["port"] = _config["backend"]["port"]
            self.RAMSTK_COM_INFO["database"] = _config["backend"]["database"]
            self.RAMSTK_COM_INFO["user"] = _config["backend"]["user"]
            self.RAMSTK_COM_INFO["password"] = _config["backend"]["password"]

        else:
            _error_msg = (
                f"Failed to read Site configuration file {self.RAMSTK_SITE_CONF}."
            )
            pub.sendMessage("fail_get_site_configuration", error_message=_error_msg)

    def set_site_configuration(self) -> None:
        """Set the site-wide RAMSTK configuration file values."""
        _dic_site_configuration: Dict[str, Any] = {
            "title": "RAMSTK Site Configuration",
            "backend": {
                "dialect": self.RAMSTK_COM_INFO["dialect"],
                "host": str(self.RAMSTK_COM_INFO["host"]),
                "port": str(self.RAMSTK_COM_INFO["port"]),
                "database": str(self.RAMSTK_COM_INFO["database"]),
                "user": str(self.RAMSTK_COM_INFO["user"]),
                "password": str(self.RAMSTK_COM_INFO["password"]),
            },
        }
        with open(self.RAMSTK_SITE_CONF, "w", encoding="utf-8") as _site_conf:
            toml.dump(_dic_site_configuration, _site_conf)

    def set_site_directories(self) -> None:
        """Set the site-wide RAMSTK directories.

        :return: None
        :rtype: None
        """
        self.RAMSTK_SITE_DIR = self._INSTALL_PREFIX + "/share/RAMSTK"
        self.RAMSTK_SITE_CONF = self.RAMSTK_SITE_DIR + "/Site.toml"

        if not file_exists(self.RAMSTK_SITE_CONF):
            self.do_create_site_configuration()


class RAMSTKUserConfiguration:  # pylint: disable=too-many-instance-attributes
    """RAMSTK configuration class.

    Attributes of the Configuration class are:

    :ivar dict RAMSTK_FORMAT_FILE: Dictionary containing the path to the format
        files to use for various widgets.  Keys are the name of the RAMSTK
        work stream module, values are the absolute path to the format file
        for that work stream module.
    :ivar dict RAMSTK_COLORS: Dictionary containing the colors to use for
        various widgets.
    :ivar dict RAMSTK_COM_INFO: Dictionary for the RAMSTK common database
        connection information.  The information contained is:

            +----------+-------------------------------+
            |   Key    | Information                   |
            +==========+===============================+
            |   host   | Host name                     |
            +----------+-------------------------------+
            |   port   | Host port                     |
            +----------+-------------------------------+
            | database | Database name                 |
            +----------+-------------------------------+
            |   user   | User name                     |
            +----------+-------------------------------+
            | password | User password                 |
            +----------+-------------------------------+

    :ivar dict RAMSTK_PROG_INFO: Dictionary for RAMSTK Program database
        connection information.  The information contained is:

            +----------+-------------------------------+
            |   Key    | Information                   |
            +==========+===============================+
            |   host   | Host name                     |
            +----------+-------------------------------+
            |   port   | Host port                     |
            +----------+-------------------------------+
            | database | Database name                 |
            +----------+-------------------------------+
            |   user   | User name                     |
            +----------+-------------------------------+
            | password | User password                 |
            +----------+-------------------------------+

    :ivar dict RAMSTK_TABPOS: Dictionary containing the location of tabs in the
        three main Gtk.Notebook() widgets.  Can be one of:

            * Top
            * Bottom
            * Left
            * Right

            +------------+---------------+----------+
            |    Key     | Notebook      | Default  |
            +============+===============+==========+
            |  listbook  | Module Book   |  *top*   |
            +------------+---------------+----------+
            | modulebook | Work Book     | *bottom* |
            +------------+---------------+----------+
            |  workbook  | List Book     | *bottom* |
            +------------+---------------+----------+

    :ivar dict RAMSTK_SEVERITY: Dictionary for failure severity categories.
    :ivar dict RAMSTK_HAZARDS: Dictionary for potential hazards.
    :ivar dict RAMSTK_REQUIREMENT_TYPES: Dictionary of requirement types.
    :ivar dict RAMSTK_RPN_SEVERITY: Dictionary for RPN Severity categories.
    :ivar dict RAMSTK_RPN_OCCURRENCE: Dictionary for RPN Occurrence categories.
    :ivar dict RAMSTK_RPN_DETECTION: Dictionary for RPN Detection categories.
    :ivar dict RAMSTK_MODULES: Dictionary of active modules in the open RAMSTK
        Program database.  Where 1 = active and 0 = inactive.  Keys are:

            * Function
            * Hardware
            * Requirements
            * Revision
            * Validation

    :ivar list RAMSTK_PAGE_NUMBER: Dictionary indicating which page each RAMSTK
        module occupies in the ModuleBook.
    :ivar list RAMSTK_RISK_POINTS: List for risk level cutoffs.  Cutoffs are:

        +-------+---------------------------+
        | Index | Risk Level Cutoff Value   |
        +=======+===========================+
        |   0   | Low to medium             |
        +-------+---------------------------+
        |   1   | Medium to high            |
        +-------+---------------------------+

    :ivar float RAMSTK_HR_MULTIPLIER: The failure rate multiplier.  All failure
        rates will be multiplied by this value for display.  This allows
        failure rates to display without using scientific notation.  Set to one
        to use scientific notation.  Default value is *1000000.0*.
    :ivar float RAMSTK_MTIME: The default mission time for new RAMSTK Programs.
    :ivar int RAMSTK_DEC_PLACES: Number of decimal places to show in numerical
        results.  Default value is *6*.
    :ivar int RAMSTK_MODE_SOURCE: Indicator variable used to determine which
        failure mode source to use.  Sources are:

            1. FMD-97
            2. MIL-STD-338

    :ivar str RAMSTK_CONF_DIR: Path to the directory containing configuration
        files used by RAMSTK.
    :ivar str RAMSTK_DATA_DIR: Path to the directory containing data files used
        by RAMSTK.
    :ivar str RAMSTK_ICON_DIR: Path to the directory containing icon files used
        by RAMSTK.
    :ivar str RAMSTK_LOG_DIR: Path to the directory containing log files used
        by RAMSTK.
    :ivar str RAMSTK_PROG_DIR: Path to the base directory containing RAMSTK
        Program database files.  This is only used when the backend is SQLite3.
    :ivar str RAMSTK_GUI_LAYOUT: Layout of the GUI to use.  Possible options
        are:

            * basic - a single window embedded with the Module Book, Work Book,
                and List Book.
            * advanced - multiple windows; one each for the Module Book, Work
                Book, and List Book.

        Default value is *basic*.
    :ivar str RAMSTK_COM_BACKEND: RAMSTK common database backend to use.
        Options are:

            * mysql (future)
            * sqlite

    :ivar str RAMSTK_BACKEND: RAMSTK Program database backend to use.  Options
        are:

            * mysql (future)
            * sqlite

    :ivar str RAMSTK_LOCALE: The language locale to use with RAMSTK.  Default
        value is *en_US*.
    :ivar str RAMSTK_OS: The operating system RAMSTK is currently running on.
    """

    # pylint: disable=too-many-statements
    def __init__(self) -> None:
        """Class for user-specific RAMSTK configuration settings."""
        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_colors = [
            "allocationbg",
            "allocationfg",
            "fmeabg",
            "fmeafg",
            "failure_definitionbg",
            "failure_definitionfg",
            "functionbg",
            "functionfg",
            "hardwarebg",
            "hardwarefg",
            "hazardbg",
            "hazardfg",
            "pofbg",
            "poffg",
            "revisionbg",
            "revisionfg",
            "requirementfg",
            "requirementbg",
            "validationbg",
            "validationfg",
            "similar_itembg",
            "similar_itemfg",
            "stakeholderbg",
            "stakeholderfg",
        ]
        self._lst_format_files = [
            "allocation",
            "failure_definition",
            "fmea",
            "function",
            "hardware",
            "hazard",
            "pof",
            "requirement",
            "revision",
            "similar_item",
            "stakeholder",
            "usage_profile",
            "validation",
        ]
        self._lst_categories = [
            "integratedcircuit",
            "semiconductor",
            "resistor",
            "capacitor",
            "inductor",
            "relay",
            "switch",
            "connection",
            "meter",
            "miscellaneous",
        ]

        # Initialize private scalar attributes.
        self._INSTALL_PREFIX = get_install_prefix()
        self._data_sub_dir = "/layouts"
        self._icon_sub_dir = "/icons"
        self._logs_sub_dir = "/logs"

        # Initialize public dictionary attributes.
        self.RAMSTK_ACTION_CATEGORY: Dict[str, Tuple[str, str, str, str]] = {}
        self.RAMSTK_ACTION_STATUS: Dict[str, Tuple[str, str, str]] = {}
        self.RAMSTK_AFFINITY_GROUPS: Dict[str, Tuple[str, str]] = {}  # User.
        self.RAMSTK_CATEGORIES: Dict[str, str] = {}  # Static.
        self.RAMSTK_DAMAGE_MODELS: Dict[str, str] = {}  # User.
        self.RAMSTK_DETECTION_METHODS: Dict[str, Tuple[str, str, str]] = {}
        self.RAMSTK_FAILURE_MODES: Dict[str, str] = {}  # User.
        self.RAMSTK_HAZARDS: Dict[str, Tuple[str, str]] = {}  # User.
        self.RAMSTK_INCIDENT_CATEGORY: Dict[str, Tuple[str, str, str, str]] = {}
        self.RAMSTK_INCIDENT_STATUS: Dict[str, Tuple[str, str, str]] = {}
        self.RAMSTK_INCIDENT_TYPE: Dict[str, Tuple[str, str, str]] = {}
        self.RAMSTK_LOAD_HISTORY: Dict[int, str] = {}  # User.
        self.RAMSTK_MANUFACTURERS: Dict[str, Tuple[str, str, str]] = {}  # User.
        self.RAMSTK_MEASURABLE_PARAMETERS: Dict[int, Tuple[str, str, str]] = {}  # User.
        self.RAMSTK_MEASUREMENT_UNITS: Dict[str, Tuple[str, str, str]] = {}  # Admin.
        self.RAMSTK_MODULES: Dict[str, str] = {}  # Static.
        self.RAMSTK_REQUIREMENT_TYPE: Dict[str, Tuple[str, str, str]] = {}
        self.RAMSTK_RPN_DETECTION: Dict[int, str] = {}  # User.
        self.RAMSTK_RPN_OCCURRENCE: Dict[int, str] = {}  # User.
        self.RAMSTK_RPN_SEVERITY: Dict[int, str] = {}  # User.
        self.RAMSTK_SEVERITY: Dict[str, Tuple[str, str, str, str]] = {}  # Admin
        self.RAMSTK_STAKEHOLDERS: Dict[str, str] = {}  # User.
        self.RAMSTK_STRESS_LIMITS: Dict[int, List[float]] = {}  # User.
        self.RAMSTK_SUBCATEGORIES: Dict[str, Dict[str, str]] = {}  # Static.
        self.RAMSTK_USERS: Dict[str, Tuple[str, str, str, str, str]] = {}  # Admin.
        self.RAMSTK_VALIDATION_TYPE: Dict[str, Tuple[str, str, str]] = {}  # Admin.

        self.RAMSTK_COLORS: Dict[str, str] = {}
        self.RAMSTK_FORMAT_FILE: Dict[str, str] = {}
        self.RAMSTK_PAGE_NUMBER: Dict[int, str] = {
            0: "revision",
            1: "function",
            2: "requirement",
            3: "hardware",
            4: "validation",
        }
        self.RAMSTK_PROG_INFO: Dict[str, str] = {
            "dialect": "",
            "host": "",
            "port": "",
            "database": "",
            "user": "",
            "password": "",
        }
        self.RAMSTK_TABPOS = {
            "listbook": "top",
            "modulebook": "bottom",
            "workbook": "bottom",
        }
        self.RAMSTK_WORKGROUPS: Dict[str, Tuple[str, str]] = {}  # Admin.

        # Initialize public list attributes.
        self.RAMSTK_FAILURE_PROBABILITY = [
            [_("Level E - Extremely Unlikely"), 1],
            [_("Level D - Remote"), 2],
            [_("Level C - Occasional"), 3],
            [_("Level B - Reasonably Probable"), 4],
            [_("Level A - Frequent"), 5],
        ]
        self.RAMSTK_RISK_POINTS = [4, 10]

        # Initialize public scalar attributes.
        self.RAMSTK_MODE = ""
        self.RAMSTK_MODE_SOURCE = 1  # 1=FMD-97
        self.RAMSTK_BACKEND = ""
        self.RAMSTK_REPORT_SIZE = "letter"
        self.RAMSTK_HR_MULTIPLIER = 1.0
        self.RAMSTK_DEC_PLACES = 6
        self.RAMSTK_MTIME = 100.0
        self.RAMSTK_GUI_LAYOUT = "advanced"
        self.RAMSTK_METHOD = "STANDARD"  # STANDARD or LRM
        self.RAMSTK_LOCALE = "en_US.UTF8"
        self.RAMSTK_LOGLEVEL = "INFO"
        if sys.platform in ["linux", "linux2"]:
            self.RAMSTK_OS = "Linux"
            self.RAMSTK_CONF_DIR = self._INSTALL_PREFIX + "/share/RAMSTK"
            self.RAMSTK_HOME_DIR = environ["HOME"]
        elif sys.platform == "win32":
            self.RAMSTK_OS = "Windows"
            self.RAMSTK_CONF_DIR = environ["PYTHONPATH"] + "/RAMSTK"
            self.RAMSTK_HOME_DIR = environ["USERPROFILE"]

        self.RAMSTK_DATA_DIR = self.RAMSTK_CONF_DIR + self._data_sub_dir
        self.RAMSTK_ICON_DIR = self.RAMSTK_CONF_DIR + self._icon_sub_dir
        self.RAMSTK_LOG_DIR = self.RAMSTK_CONF_DIR + self._logs_sub_dir
        self.RAMSTK_PROG_DIR = self.RAMSTK_HOME_DIR + "/analyses/ramstk/"

        self.RAMSTK_PROG_CONF = self.RAMSTK_CONF_DIR + "/RAMSTK.toml"
        self.RAMSTK_USER_LOG = self.RAMSTK_LOG_DIR + "/ramstk_run.log"
        self.RAMSTK_IMPORT_LOG = self.RAMSTK_LOG_DIR + "/ramstk_import.log"

        self.loaded = False

    def _do_make_configuration_dir(self) -> None:
        """Create the user configuration directory.

        :return: None
        :rtype: None
        """
        self.RAMSTK_CONF_DIR = self.RAMSTK_HOME_DIR + "/.config/RAMSTK"
        if not dir_exists(self.RAMSTK_CONF_DIR):
            try:
                makedirs(self.RAMSTK_CONF_DIR)
                self.RAMSTK_PROG_CONF = self.RAMSTK_CONF_DIR + "/RAMSTK.toml"
            except OSError:
                _error_msg = (
                    f"User's configuration directory {self.RAMSTK_CONF_DIR} does not "
                    f"exist and could not be created when attempting to create a new "
                    f"user configuration file."
                )
                pub.sendMessage(
                    "fail_create_user_configuration", error_message=_error_msg
                )

    def _do_make_data_dir(self) -> None:
        """Create the user data directory.

        :return: None
        :rtype: None
        """
        self.RAMSTK_DATA_DIR = self.RAMSTK_CONF_DIR + self._data_sub_dir
        if not dir_exists(self.RAMSTK_DATA_DIR):
            try:
                makedirs(self.RAMSTK_DATA_DIR)
            except OSError:
                _error_msg = (
                    f"User's data directory {self.RAMSTK_DATA_DIR} does not exist and "
                    f"could not be created when attempting to create a new user "
                    f"configuration file."
                )
                pub.sendMessage(
                    "fail_create_user_configuration", error_message=_error_msg
                )

    def _do_make_icon_dir(self) -> None:
        """Create the user icon directory.

        :return: None
        :rtype: None
        """
        self.RAMSTK_ICON_DIR = self.RAMSTK_CONF_DIR + self._icon_sub_dir

        if not dir_exists(self.RAMSTK_ICON_DIR):
            try:
                makedirs(self.RAMSTK_ICON_DIR)
            except OSError:
                _error_msg = (
                    f"User's icon directory {self.RAMSTK_ICON_DIR} does not exist and "
                    f"could not be created when attempting to create a new user "
                    f"configuration file."
                )
                pub.sendMessage(
                    "fail_create_user_configuration", error_message=_error_msg
                )

    def _do_make_log_dir(self) -> None:
        """Create the user log directory.

        :return: None
        :rtype: None
        """
        self.RAMSTK_LOG_DIR = self.RAMSTK_CONF_DIR + self._logs_sub_dir

        if not dir_exists(self.RAMSTK_LOG_DIR):
            try:
                makedirs(self.RAMSTK_LOG_DIR)
            except OSError:
                _error_msg = (
                    f"User's log directory {self.RAMSTK_LOG_DIR} does not exist and "
                    f"could not be created when attempting to create a new user "
                    f"configuration file."
                )
                pub.sendMessage(
                    "fail_create_user_configuration", error_message=_error_msg
                )

    def _do_make_program_dir(self) -> None:
        """Create the user program directory.

        :return: None
        :rtype: None
        """
        if not dir_exists(self.RAMSTK_PROG_DIR):
            try:
                makedirs(self.RAMSTK_PROG_DIR)
            except OSError:
                _error_msg = (
                    f"Program directory {self.RAMSTK_PROG_DIR} does not exist and "
                    f"could not be created when attempting to create a new user "
                    f"configuration file."
                )
                pub.sendMessage(
                    "fail_create_user_configuration", error_message=_error_msg
                )

    def do_create_user_configuration(self) -> None:
        """Create the default user configuration file.

        :return: None
        :rtype: None
        """
        # Create the directories needed for the user.  Always prefer the RAMSTK
        # directories in the user's $HOME over the system-wide directories.
        # Configuration directory.
        self._do_make_configuration_dir()
        self._do_make_data_dir()
        self._do_make_icon_dir()
        self._do_make_log_dir()
        self._do_make_program_dir()

        # Copy format files from RAMSTK_SITE_DIR (system) to the user's
        # RAMSTK_CONF_DIR.
        for _file in glob.glob(self._INSTALL_PREFIX + "/share/RAMSTK/layouts/*.toml"):
            file_util.copy_file(_file, self.RAMSTK_DATA_DIR)

        # Copy the icons from RAMSTK_SITE_DIR (system) to the user's
        # RAMSTK_ICON_DIR.
        try:
            dir_util.copy_tree(
                self._INSTALL_PREFIX + "/share/RAMSTK/icons/", self.RAMSTK_ICON_DIR
            )
        except DistutilsFileError:
            _error_msg = (
                f"Attempt to copy RAMSTK icons from site-wide icon directory "
                f"{self._INSTALL_PREFIX}/share/RAMSTK/icons/ to user's icon "
                f"directory {self.RAMSTK_ICON_DIR} failed."
            )
            pub.sendMessage("fail_copy_icons_to_user", error_message=_error_msg)

        # Create the default RAMSTK user configuration file.
        _dic_user_configuration = {
            "title": "RAMSTK User Configuration",
            "general": {
                "firstrun": "True",
                "reportsize": "letter",
                "frmultiplier": "1000000.0",
                "calcreltime": "100.0",
                "decimal": "6",
                "modesource": "1",
                "moduletabpos": "top",
                "listtabpos": "bottom",
                "worktabpos": "bottom",
                "loglevel": "INFO",
            },
            "backend": {
                "dialect": "postgres",
                "host": "localhost",
                "port": "5432",
                "database": "",
                "user": "",
                "password": "",
            },
            "directories": {
                "datadir": self.RAMSTK_DATA_DIR,
                "icondir": self.RAMSTK_ICON_DIR,
                "logdir": self.RAMSTK_LOG_DIR,
                "progdir": self.RAMSTK_PROG_DIR,
            },
            "layouts": {
                "allocation": "allocation.toml",
                "failure_definition": "failure_definition.toml",
                "fmea": "fmea.toml",
                "function": "function.toml",
                "hardware": "hardware.toml",
                "hazard": "hazops.toml",
                "pof": "pof.toml",
                "requirement": "requirement.toml",
                "revision": "revision.toml",
                "similar_item": "similar_item.toml",
                "stakeholder": "stakeholder.toml",
                "usage_profile": "usage_profile.toml",
                "validation": "validation.toml",
            },
            "colors": {
                "allocationbg": "#FFFFFF",
                "allocationfg": "#000000",
                "failure_definitionbg": "#FFFFFF",
                "failure_definitionfg": "#000000",
                "fmeabg": "#FFFFFF",
                "fmeafg": "#000000",
                "functionbg": "#FFFFFF",
                "functionfg": "#000000",
                "hardwarebg": "#FFFFFF",
                "hardwarefg": "#000000",
                "hazardbg": "#FFFFFF",
                "hazardfg": "#000000",
                "pofbg": "#FFFFFF",
                "poffg": "#000000",
                "requirementbg": "#FFFFFF",
                "requirementfg": "#000000",
                "revisionbg": "#FFFFFF",
                "revisionfg": "#000000",
                "similar_itembg": "#FFFFFF",
                "similar_itemfg": "#000000",
                "stakeholderbg": "#FFFFFF",
                "stakeholderfg": "#000000",
                "validationbg": "#FFFFFF",
                "validationfg": "#000000",
            },
            "stress": {
                "integratedcircuit": [
                    0.8,
                    0.9,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                    0.0,
                    0.0,
                    125.0,
                    125.0,
                ],
                "semiconductor": [1.0, 1.0, 0.7, 0.9, 1.0, 1.0, 0.0, 0.0, 125.0, 125.0],
                "resistor": [1.0, 1.0, 0.5, 0.9, 1.0, 1.0, 0.0, 0.0, 125.0, 125.0],
                "capacitor": [1.0, 1.0, 1.0, 1.0, 0.6, 0.9, 10.0, 0.0, 125.0, 125.0],
                "inductor": [0.6, 0.9, 1.0, 1.0, 0.5, 0.9, 15.0, 0.0, 125.0, 125.0],
                "relay": [0.75, 0.9, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 125.0, 125.0],
                "switch": [0.75, 0.9, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 125.0, 125.0],
                "connection": [0.7, 0.9, 1.0, 1.0, 0.7, 0.9, 25.0, 0.0, 125.0, 125.0],
                "meter": [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 125.0, 125.0],
                "miscellaneous": [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 125.0, 125.0],
            },
        }

        try:
            with open(self.RAMSTK_PROG_CONF, "w", encoding="utf-8") as _user_conf:
                toml.dump(_dic_user_configuration, _user_conf)
            pub.sendMessage("succeed_create_user_configuration")
        except TypeError:
            _error_msg = (
                f"User configuration file {self.RAMSTK_PROG_CONF} is not a file."
            )
            pub.sendMessage("fail_create_user_configuration", error_message=_error_msg)

    def get_user_configuration(self) -> None:
        """Read the RAMSTK user configuration file.

        :return: None
        :rtype: None
        """
        # Try to read the user's configuration file.  If it doesn't exist,
        # create a new one.  If those options fail, read the system-wide
        # configuration file and keep going.
        if file_exists(self.RAMSTK_PROG_CONF):
            _config = toml.load(self.RAMSTK_PROG_CONF)

            for _color in self._lst_colors:
                self.RAMSTK_COLORS[_color] = _config["colors"][_color]

            for _file in self._lst_format_files:
                self.RAMSTK_FORMAT_FILE[_file] = _config["layouts"][_file]

            # Hardware categories are stored as integers, but configuration
            # file keys are human-readable nouns.  This converts the noun key
            # to the equivalent integer key.
            for _category in enumerate(self._lst_categories):
                self.RAMSTK_STRESS_LIMITS[_category[0] + 1] = _config["stress"][
                    _category[1]
                ]

            self.RAMSTK_BACKEND = _config["backend"]["dialect"]
            self.RAMSTK_PROG_INFO["dialect"] = _config["backend"]["dialect"]
            self.RAMSTK_PROG_INFO["host"] = _config["backend"]["host"]
            self.RAMSTK_PROG_INFO["port"] = _config["backend"]["port"]
            self.RAMSTK_PROG_INFO["database"] = _config["backend"]["database"]
            self.RAMSTK_PROG_INFO["user"] = _config["backend"]["user"]
            self.RAMSTK_PROG_INFO["password"] = _config["backend"]["password"]

            self.RAMSTK_DATA_DIR = _config["directories"]["datadir"]
            self.RAMSTK_ICON_DIR = _config["directories"]["icondir"]
            self.RAMSTK_LOG_DIR = _config["directories"]["logdir"]
            self.RAMSTK_PROG_DIR = _config["directories"]["progdir"]

            self.RAMSTK_REPORT_SIZE = _config["general"]["reportsize"]
            self.RAMSTK_HR_MULTIPLIER = float(_config["general"]["frmultiplier"])
            self.RAMSTK_DEC_PLACES = int(_config["general"]["decimal"])
            self.RAMSTK_MTIME = float(_config["general"]["calcreltime"])
            self.RAMSTK_MODE_SOURCE = _config["general"]["modesource"]
            self.RAMSTK_TABPOS["listbook"] = _config["general"]["listtabpos"]
            self.RAMSTK_TABPOS["modulebook"] = _config["general"]["moduletabpos"]
            self.RAMSTK_TABPOS["workbook"] = _config["general"]["worktabpos"]
            self.RAMSTK_LOGLEVEL = _config["general"]["loglevel"]
            if self.RAMSTK_LOG_DIR == "":
                self.RAMSTK_USER_LOG = "./ramstk_run.log"
                self.RAMSTK_IMPORT_LOG = "./ramstk_import.log"
            else:
                self.RAMSTK_USER_LOG = self.RAMSTK_LOG_DIR + "/ramstk_run.log"
                self.RAMSTK_IMPORT_LOG = self.RAMSTK_LOG_DIR + "/ramstk_import.log"

        else:
            _error_msg = (
                f"Failed to read User configuration file {self.RAMSTK_PROG_CONF}."
            )
            pub.sendMessage("fail_get_user_configuration", error_message=_error_msg)

    def set_user_configuration(self) -> None:
        """Write changes to the user's configuration file.

        :return: None
        :rtype: None
        """
        _dic_user_configuration: Dict[str, Any] = {
            "title": "RAMSTK User Configuration",
            "general": {
                "reportsize": self.RAMSTK_REPORT_SIZE,
                "frmultiplier": str(self.RAMSTK_HR_MULTIPLIER),
                "calcreltime": str(self.RAMSTK_MTIME),
                "decimal": str(self.RAMSTK_DEC_PLACES),
                "modesource": str(self.RAMSTK_MODE_SOURCE),
                "moduletabpos": self.RAMSTK_TABPOS["modulebook"],
                "listtabpos": self.RAMSTK_TABPOS["listbook"],
                "worktabpos": self.RAMSTK_TABPOS["workbook"],
                "loglevel": self.RAMSTK_LOGLEVEL,
            },
            "backend": {
                "dialect": self.RAMSTK_PROG_INFO["dialect"],
                "host": str(self.RAMSTK_PROG_INFO["host"]),
                "port": str(self.RAMSTK_PROG_INFO["port"]),
                "database": str(self.RAMSTK_PROG_INFO["database"]),
                "user": str(self.RAMSTK_PROG_INFO["user"]),
                "password": str(self.RAMSTK_PROG_INFO["password"]),
            },
            "directories": {
                "datadir": self.RAMSTK_DATA_DIR,
                "icondir": self.RAMSTK_ICON_DIR,
                "logdir": self.RAMSTK_LOG_DIR,
                "progdir": self.RAMSTK_PROG_DIR,
            },
            "layouts": {
                "allocation": self.RAMSTK_FORMAT_FILE["allocation"],
                "failure_definition": self.RAMSTK_FORMAT_FILE["failure_definition"],
                "fmea": self.RAMSTK_FORMAT_FILE["fmea"],
                "function": self.RAMSTK_FORMAT_FILE["function"],
                "hardware": self.RAMSTK_FORMAT_FILE["hardware"],
                "hazard": self.RAMSTK_FORMAT_FILE["hazard"],
                "pof": self.RAMSTK_FORMAT_FILE["pof"],
                "requirement": self.RAMSTK_FORMAT_FILE["requirement"],
                "revision": self.RAMSTK_FORMAT_FILE["revision"],
                "similar_item": self.RAMSTK_FORMAT_FILE["similar_item"],
                "stakeholder": self.RAMSTK_FORMAT_FILE["stakeholder"],
                "usage_profile": self.RAMSTK_FORMAT_FILE["usage_profile"],
                "validation": self.RAMSTK_FORMAT_FILE["validation"],
            },
            "colors": {
                "allocationbg": self.RAMSTK_COLORS["allocationbg"],
                "allocationfg": self.RAMSTK_COLORS["allocationbg"],
                "failure_definitionbg": self.RAMSTK_COLORS["failure_definitionbg"],
                "failure_definitionfg": self.RAMSTK_COLORS["failure_definitionbg"],
                "fmeabg": self.RAMSTK_COLORS["fmeabg"],
                "fmeafg": self.RAMSTK_COLORS["fmeafg"],
                "functionbg": self.RAMSTK_COLORS["functionbg"],
                "functionfg": self.RAMSTK_COLORS["functionfg"],
                "hardwarebg": self.RAMSTK_COLORS["hardwarebg"],
                "hardwarefg": self.RAMSTK_COLORS["hardwarefg"],
                "hazardbg": self.RAMSTK_COLORS["hazardbg"],
                "hazardfg": self.RAMSTK_COLORS["hazardfg"],
                "pofbg": self.RAMSTK_COLORS["pofbg"],
                "poffg": self.RAMSTK_COLORS["poffg"],
                "requirementbg": self.RAMSTK_COLORS["requirementbg"],
                "requirementfg": self.RAMSTK_COLORS["requirementfg"],
                "revisionbg": self.RAMSTK_COLORS["revisionbg"],
                "revisionfg": self.RAMSTK_COLORS["revisionfg"],
                "similar_itembg": self.RAMSTK_COLORS["similar_itembg"],
                "similar_itemfg": self.RAMSTK_COLORS["similar_itemfg"],
                "stakeholderbg": self.RAMSTK_COLORS["stakeholderbg"],
                "stakeholderfg": self.RAMSTK_COLORS["stakeholderfg"],
                "validationbg": self.RAMSTK_COLORS["validationbg"],
                "validationfg": self.RAMSTK_COLORS["validationfg"],
            },
            "stress": {
                "integratedcircuit": self.RAMSTK_STRESS_LIMITS[1],
                "semiconductor": self.RAMSTK_STRESS_LIMITS[2],
                "resistor": self.RAMSTK_STRESS_LIMITS[3],
                "capacitor": self.RAMSTK_STRESS_LIMITS[4],
                "inductor": self.RAMSTK_STRESS_LIMITS[5],
                "relay": self.RAMSTK_STRESS_LIMITS[6],
                "switch": self.RAMSTK_STRESS_LIMITS[7],
                "connection": self.RAMSTK_STRESS_LIMITS[8],
                "meter": self.RAMSTK_STRESS_LIMITS[9],
                "miscellaneous": self.RAMSTK_STRESS_LIMITS[10],
            },
        }

        with open(self.RAMSTK_PROG_CONF, "w", encoding="utf-8") as _user_conf:
            toml.dump(_dic_user_configuration, _user_conf)

    def set_user_directories(self) -> None:
        """Set the user-specific configuration directories.

        :return: None
        :rtype: None
        """
        # Prefer user-specific directories in their $HOME directory over the
        # system-wide directories.
        if dir_exists(self.RAMSTK_HOME_DIR + "/.config/RAMSTK"):
            self.RAMSTK_CONF_DIR = self.RAMSTK_HOME_DIR + "/.config/RAMSTK"
        else:
            self.RAMSTK_CONF_DIR = self._INSTALL_PREFIX + "/share/RAMSTK"

        self.RAMSTK_PROG_CONF = self.RAMSTK_CONF_DIR + "/RAMSTK.toml"

        if dir_exists(self.RAMSTK_CONF_DIR + self._data_sub_dir):
            self.RAMSTK_DATA_DIR = self.RAMSTK_CONF_DIR + self._data_sub_dir
        else:
            self.RAMSTK_DATA_DIR = self._INSTALL_PREFIX + "/share/RAMSTK/layouts"

        if dir_exists(self.RAMSTK_CONF_DIR + self._icon_sub_dir):
            self.RAMSTK_ICON_DIR = self.RAMSTK_CONF_DIR + self._icon_sub_dir
        else:
            self.RAMSTK_ICON_DIR = self._INSTALL_PREFIX + "/share/RAMSTK/icons"

        if dir_exists(self.RAMSTK_CONF_DIR + self._logs_sub_dir):
            self.RAMSTK_LOG_DIR = self.RAMSTK_CONF_DIR + self._logs_sub_dir
        else:
            self.RAMSTK_LOG_DIR = self._INSTALL_PREFIX + "/share/RAMSTK/logs"

        if dir_exists(self.RAMSTK_HOME_DIR + "/analyses/ramstk"):
            self.RAMSTK_PROG_DIR = self.RAMSTK_HOME_DIR + "/analyses/ramstk"
        else:
            self.RAMSTK_PROG_DIR = self.RAMSTK_HOME_DIR
